import logging
from typing import Type, TypeVar, List, Union
import swibots
from swibots.api.chat.models import Message


class DeleteMessage:
    async def delete_message(self: "swibots.ApiClient", message: int | Message) -> bool:
        """Delete a message

        Parameters:
            message (``int`` | ``~switch.api.chat.models.Message``): The message to delete

        Returns:
            ``bool``: Whether the message was deleted

        Raises:
            ``~switch.error.SwitchError``: If the message could not be deleted

        This method does the same as :meth:`~switch.api.chat.controllers.MessageController.delete_message`.
        """
        logging.warning(
            "This method is deprecated and is going to be removed in the future! Use client.delete_messages to achieve same results!"
        )
        return await self.chat_service.messages.delete_messages([message])

    async def delete_messages(
        self: "swibots.ApiClient", message_ids: List[Union[int, Message]]
    ) -> bool:
        """Delete one or more messages.

        This method allows you to delete one or more messages from a chat.

        Parameters:
            message_ids (List[Union[int, Message]]): A list of message IDs or Message objects to delete.

        Returns:
            bool: True if the messages were deleted successfully, False otherwise.

        Raises:
            SwitchError: If the messages could not be deleted.

        Note:
            This method internally calls the :meth:`~switch.api.chat.controllers.MessageController.delete_message` method.

        Example:
            >>> client = Client(...)
            >>> message_ids = [1, 2, 3]
            >>> result = await client.delete_messages(message_ids)
            >>> print(result)
            True
        """
        return await self.chat_service.messages.delete_messages(message_ids=message_ids)



from io import BytesIO
from swibots.base import SwitchObject
from swibots.utils.types import JSONDict
from ....api.common.models import Media
from typing import List, Optional, Union

import swibots


class Sticker(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        id: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        emoji: Optional[str] = None,
        sticker_pack_id: Optional[str] = None,
        created_by: Optional[int] = None,
        media_id: Optional[int] = None,
        sticker_info: Optional[Media] = None,
    ):
        super().__init__(app)
        self.id = id
        self.name = name
        self.description = description
        self.emoji = emoji
        self.sticker_pack_id = sticker_pack_id
        self.sticker_info = sticker_info
        self.created_by = created_by
        self.media_id = media_id

    def from_json(self, data: dict = None) -> "Sticker":
        if data is not None:
            self.id = data.get("id")
            self.name = data.get("name")
            self.description = data.get("description")
            self.emoji = data.get("linkedEmoji")
            self.sticker_info = Media.build_from_json(
                data.get("stickerMediaInfo", self.sticker_info)
            )
            self.sticker_pack_id = data.get("stickerPackId")
            self.created_by = int(data.get("createdBy", 0))
            self.media_id = data.get("stickerMediaId")
        return self

    def to_json(self) -> dict:
        return {
            "id": self.id,
            "name": self.name,
            "description": self.description,
            "linkedEmoji": self.emoji,
            "stickerPackId": self.sticker_pack_id,
            "createdBy": self.created_by,
            "stickerMediaId": self.media_id,
            "stickerMediaInfo": (
                self.sticker_info.to_json() if self.sticker_info else None
            ),
        }

    async def delete(self) -> bool:
        return await self.app.delete_sticker(self.id)


class StickerPack(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        id: Optional[str] = None,
        name: Optional[str] = None,
        pack_type: Optional[str] = None,
        created_by: Optional[int] = None,
        access_control: Optional[str] = None,
        thumb_media_id: Optional[int] = None,
        thumb_info: Optional[Media] = None,
    ):
        super().__init__(app)
        self.id = id
        self.name = name
        self.created_by = created_by
        self.thumb_media_id = thumb_media_id
        self.thumb_info = thumb_info
        self.access_control = access_control
        self.pack_type = pack_type
        self.stickers: List[Sticker] = None

    def from_json(self, data: JSONDict = None) -> "StickerPack":
        if data is not None:
            self.id = data.get("id")
            self.name = data.get("name")
            self.created_by = int(data.get("createdBy", 0))
            self.pack_type = data.get("packType")
            self.thumb_media_id = data.get("thumbnailMediaId")
            self.thumb_info = Media.build_from_json(
                data.get("thumbnailMediaInfo"), self.app
            )
            self.stickers = data.get("sortedStickers")
        return self

    def to_json_request(self) -> JSONDict:
        return {
            "name": self.name,
            "id": self.id,
            "accessControl": self.access_control,
            "sortedStickers": self.stickers,
            "packType": self.pack_type,
        }

    async def delete(self) -> bool:
        return await self.app.delete_sticker_pack(self.id)

    async def add(
        self, sticker: str | BytesIO, name: str, description: str, emoji: str
    ) -> Sticker:
        return await self.app.create_sticker(
            sticker, name, description, emoji, pack_id=self.id
        )



from typing import Dict, Any
from swibots.base import SwitchObject


class CallbackResponse(SwitchObject):

    def __init__(self):
        self.search_query = None
        self.file_url = None
        self.file_name = None
        self.input_value = None
        self.parent_id = None
        self.new_url = None
        self.__data = None

    def get(self, key: str):
        if self.__data and (val := self.__data.get(key)):
            return val
        return self.to_json().get(key)

    def from_json(self, data: Dict[str, Any] | None) -> Any:
        if data is not None:
            self.search_query = data.get("searchQuery")
            self.file_url = data.get("fileResponse")
            self.file_name = data.get("fileName")
            self.parent_id = data.get("callbackQueryId")
            self.input_value = data.get("inputValue")
            self.new_url = data.get("url")
        self.__data = data
        return self

    def to_json(self) -> Dict[str, Any]:
        return {
            "searchQuery": self.search_query,
            "fileResponse": self.file_url,
            "fileName": self.file_name,
            "callbackQueryId": self.parent_id,
            "inputValue": self.input_value,
            "url": self.new_url,
        }



from typing import Type, TypeVar, Optional, List
import swibots
from io import BytesIO
from swibots.api.chat.models import Sticker, StickerPack


class StickerMethods:
    async def create_sticker(
        self: "swibots.ApiClient",
        sticker: str | BytesIO,
        name: str,
        description: str,
        emoji: str,
        pack_id: str,
    ) -> Sticker:
        """Create Sticker

        Args:
            sticker (str | BytesIO): path to file or BytesIO Object with .name
            name (str): Name of the sticker
            description (str): Description of the sticker.
            emoji (str): emoji linked with sticker.
            pack_id (str): pack Id to which sticker belongs.

        Returns:
            Sticker: the sticker object.
        """
        return await self.chat_service.stickers.create_sticker(
            sticker=sticker,
            name=name,
            description=description,
            emoji=emoji,
            pack_id=pack_id,
        )

    async def get_stickers(
        self: "swibots.ApiClient",
        pack_id: str,
        limit: Optional[int] = 30,
        offset: Optional[int] = 0,
    ) -> List[Sticker]:
        """Get All stickers from the pack

        Args:
            pack_id (str): Pack id of sticker.
            limit (Optional[int], optional): limit . Defaults to 30.
            offset (Optional[int], optional): offset to fetch. Defaults to 0.

        Returns:
            List[Sticker]
        """
        return await self.chat_service.stickers.get_stickers(
            pack_id, limit=limit, offset=offset
        )

    async def create_sticker_pack(
        self: "swibots.ApiClient",
        name: str,
        pack_type: str,
        access: str = "GLOBAL",
        thumb: Optional[str | BytesIO] = None,
    ) -> StickerPack:
        """Create Sticker pack

        Args:
            name (str): Name of sticker pack.
            pack_type (str): pack type for the sticker pack (STATIC, ANIMATED, VIDEO).
            access (str, optional): access mode for the pack. Defaults to "GLOBAL".
            thumb (Optional[str | BytesIO], optional): path to sticker pack thumbnail. Defaults to None.

        Returns:
            StickerPack: the resultant StickerPack
        """
        return await self.chat_service.stickers.create_sticker_pack(
            name=name, pack_type=pack_type, access=access, thumb=thumb
        )

    async def delete_sticker(self: "swibots.ApiClient", sticker_id: str) -> bool:
        """Delete sticker

        Args:
            sticker_id (str): Sticker ID

        Returns:
            bool: whether sticker was deleted
        """
        return await self.chat_service.stickers.delete_sticker(sticker_id)

    async def delete_sticker_pack(self: "swibots.ApiClient", pack_id: str) -> bool:
        """Delete sticker pack

        Args:
            pack_id (str): Pack ID

        Returns:
            bool: whether sticker pack was deleted
        """
        return await self.chat_service.stickers.delete_sticker_pack(pack_id)

    async def search_sticker_packs(
        self: "swibots.ApiClient",
        query: str,
        limit: Optional[int] = 20,
        offset: Optional[int] = 0,
    ) -> List[StickerPack]:
        """Search Sticker Packs

        Args:
            query (str): query to search
            limit (Optional[int], optional): Defaults to 20.
            offset (Optional[int], optional):  Defaults to 0.

        Returns:
            List[StickerPack]: List of sticker packs.
        """
        return await self.chat_service.stickers.search_sticker_packs(
            query=query, limit=limit, offset=offset
        )

    async def get_all_sticker_packs(
        self: "swibots.ApiClient", limit: Optional[int] = 20, offset: Optional[int] = 0
    ) -> List[StickerPack]:
        """Get All Sticker packs

        Args:
            limit (Optional[int], optional): Defaults to 20.
            offset (Optional[int], optional): Defaults to 0.

        Returns:
            List[StickerPack]
        """
        return await self.chat_service.stickers.get_all_sticker_packs(
            limit=limit, offset=offset
        )

    async def sort_stickers(
        self: "swibots.ApiClient", pack: StickerPack, sorted_stickers: List[str]
    ) -> StickerPack:
        """Sort stickers in sticker pack

        Args:
            pack (StickerPack): Sticker Pack
            sorted_stickers (List[str]): Sorted stickers.

        Returns:
            StickerPack: Sticker pack with sorted stickers.
        """
        return await self.chat_service.stickers.sort_stickers(
            pack=pack, sorted_stickers=sorted_stickers
        )

    async def search_sticker_packs(
        self: "swibots.ApiClient", query: str, offset: int = 0, limit: int = 10
    ) -> List[StickerPack]:
        """Search sticker packs by query

        Args:
            query (str): _description_
            offset (int, optional): _description_. Defaults to 0.
            limit (int, optional): Defaults to 10.

        """
        return await self.chat_service.stickers.search_sticker_packs(
            query=query, limit=limit, offset=offset
        )

    async def install_sticker_set(
        self: "swibots.ApiClient", pack_id: str, community_id: str = None
    ) -> bool:
        """Install sticker set from pack Id

        Args:
            pack_id (str): Sticker pack id.
            community_id (str, optional): community id to link. Defaults to None.
        """
        return await self.chat_service.stickers.install_sticker_pack(
            pack_id, community_id
        )



import logging
from typing import TYPE_CHECKING, List
from swibots.api.community.models import InstantMessaging

if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community/instant/messaging"


class InstantMessagingController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def enable_messages(
        self, community_id: str, group_id: str, bot_id: str | int = None
    ) -> InstantMessaging:
        if not bot_id:
            bot_id = self.client.user.id
        response = await self.client.get(
            f"{BASE_PATH}/enable",
            data={"communityId": community_id, "groupId": group_id, "botId": bot_id},
        )
        return self.client.build_object(InstantMessaging, response.data.get("result"))

    async def disable_messages(
        self, community_id: str, group_id: str, bot_id: str | int = None
    ) -> InstantMessaging:
        if not bot_id:
            bot_id = self.client.user.id
        response = await self.client.get(
            f"{BASE_PATH}/disable",
            data={"communityId": community_id, "groupId": group_id, "botId": bot_id},
        )
        return self.client.build_object(InstantMessaging, response.data.get("result"))

    async def get_messaging_enabled(self, community_id: str, group_id: str) -> List[InstantMessaging]:
        resp = await self.client.get(f"{BASE_PATH}/bots?communityId={community_id}&groupId={group_id}")
        return self.client.build_list(InstantMessaging, resp.data.get("result"))


from typing import Optional, Any
from swibots.base import SwitchObject
from swibots.api.common.models import User
from swibots.utils.types import JSONDict


class Tournament(SwitchObject):
    def __init__(
        self,
        id: Optional[str] = None,
        community_id: Optional[str] = None,
        creator: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        cover_img: Optional[str] = None,
        status: Optional[str] = None,
        start_time: Optional[Any] = None,
        end_time: Optional[Any] = None,
        prizes: Optional[Any] = None,
        link: Optional[Any] = None,
        xp: Optional[str] = None,
        shortlist_min_cap: Optional[str] = None,
        state: Optional[str] = None,
    ):
        self.id = id
        self.community_id = community_id
        self.creator = creator
        self.name = name
        self.description = description
        self.cover_img = cover_img
        self.start_time = start_time
        self.end_time = end_time
        self.link = link
        self.xp = xp
        self.prizes = prizes
        self.status = status
        self.shortlist_min_cap = shortlist_min_cap
        self.state = state

    def to_json(self) -> JSONDict:
        return {
            "id": self.id,
            "community_id": self.community_id,
            "creator": self.creator,
            "cover_img": self.cover_img,
            "name": self.name,
            "description": self.description,
            "start_time": self.start_time,
            "end_time": self.end_time,
            "state": self.state,
            "status": self.status,
            "shortlist_min_cap": self.shortlist_min_cap,
            "link": self.link,
            "prizes": self.prizes,
            "xp": self.xp,
        }



from typing import Optional
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject


class InstantMessaging(SwitchObject):
    def __init__(
        self,
        id: Optional[str] = None,
        community_id: Optional[str] = None,
        group_id: Optional[str] = None,
        channel_id: Optional[str] = None,
        bot_id: Optional[int] = None,
        enabled: Optional[bool] = False,
    ):
        self.id = id
        self.community_id = community_id
        self.group_id = group_id
        self.channel_id = channel_id
        self.bot_id = bot_id
        self.enabled = enabled

    def from_json(self, data: JSONDict = None) -> "InstantMessaging":
        if data is not None:
            self.id = data.get("id")
            self.community_id = data.get("communityId")
            self.group_id = data.get("groupId")
            self.channel_id = data.get("channelId")
            self.bot_id = data.get("botId")
            self.enabled = data.get("enabled")
        return self



import swibots

from typing import Optional, List

from swibots.responses import CommunityQuestResponse
from swibots.api.community.models import Quest, QuestCategory
from swibots.api.common.models import User


class QuestsMethods:
    """
    Methods for managing permissions in a community.

    Args:
        client (swibots.ApiClient): The API client.
    """

    async def create_quest(self: "swibots.ApiClient", quest: Quest) -> Quest:
        """Create a quest

        Args:
            quest (Quest): the reference to quest

        Returns:
            `Quest`: the resultant
        """
        return await self.community_service.quests.create_quest(quest)

    async def update_quest(self: "swibots.ApiClient", quest: Quest):
        """Update a quest

        Args:
            quest (Quest): the reference to quest

        Returns:
            `Quest`: the updated quest
        """
        return await self.community_service.quests.update_quest(quest=quest)

    async def get_quest(self: "swibots.ApiClient", quest_id: int) -> Quest:
        """Get quest by quest id

        Args:
            quest_id (int): Quest ID

        Returns:
            `Quest`: the resulting quest.
        """
        return await self.community_service.quests.get_quest(quest_id)

    async def submit_quest(
        self: "swibots.ApiClient", quest_id: int, community_id: str, description: str
    ):
        """Submit quest in the community

        Args:
            quest_id (int): Quest ID.
            community_id (str): Community ID.
            description (str): Description

        Returns:
            _type_: _description_
        """
        return await self.community_service.quests.submit_quest(
            quest_id=quest_id, community_id=community_id, description=description
        )

    async def approve_quest(
        self: "swibots.ApiClient", community_id: str, quest_id: int, winner_id: int
    ) -> Quest:
        """Select a winner for the quest.

        Args:
            community_id (str): Community ID
            quest_id (int): Quest ID.
            winner_id (int): User id of winner.

        Returns:
            Quest: the resulting quest
        """
        return await self.community_service.quests.approve_quest(
            community_id=community_id, quest_id=quest_id, winner_id=winner_id
        )

    async def delete_quest(self: "swibots.ApiClient", quest_id: str) -> bool:
        """Delete a quest by quest id

        Args:
            quest_id (str): Quest ID

        Returns:
            bool: whether quest was deleted or not
        """
        return await self.community_service.quests.delete_quest(quest_id=quest_id)

    async def get_quest_participants(
        self: "swibots.ApiClient", quest_id: int
    ) -> List[User]:
        """Get Participants of the quest by quest id.

        Args:
            quest_id (int): Quest ID

        Returns:
            List[User]
        """
        return await self.community_service.quests.get_quest_participants(
            quest_id=quest_id
        )

    async def get_quests_by_community(self: "swibots.ApiClient", community_id: str):
        """Get Quests by community id

        Args:
            community_id (str): _description_

        Returns:
            `CommunityQuestResponse`
        """
        return await self.community_service.quests.get_quests_by_community(
            community_id=community_id
        )

    async def get_quest_categories(
        self: "swibots.ApiClient", community_id: str
    ) -> List[QuestCategory]:
        """Get quests categories by community id.

        Args:
            community_id (str): Community ID.

        Returns:
            List[QuestCategory]: List of quest categories.
        """
        return await self.community_service.quests.get_quest_categories(
            community_id=community_id
        )

    async def get_quest_category(
        self: "swibots.ApiClient", category_id: str
    ) -> QuestCategory:
        """Get quest category by category id.

        Args:
            category_id (str): Category ID.

        Returns:
            QuestCategory
        """
        return await self.community_service.quests.get_quest_category(
            category_id=category_id
        )

    async def create_quest_category(
        self: "swibots.ApiClient", category_name: str, category_id: Optional[str] = None
    ) -> QuestCategory:
        """Create quest category

        Args:
            category_name (str): Category Name
            category_id (Optional[str], optional): Category ID. Defaults to `category_name`

        Returns:
            QuestCategory:
        """
        return await self.community_service.quests.create_quest_category(
            category_name=category_name, category_id=category_id
        )

    async def delete_quest_category(
        self: "swibots.ApiClient", category_id: str
    ) -> bool:
        """Delete Quest category by category ID.

        Args:
            category_id (str): Category ID

        Returns:
            bool: Whether category was deleted or not.
        """
        return await self.community_service.quests.delete_quest_category(
            category_id=category_id
        )



import swibots
from swibots.base import SwitchObject
from typing import List


class SessionUser(SwitchObject):
    def __init__(
        self,
        app: "swibots.Client" = None,
        session_id: str = None,
        user_id: int = None,
        name: str = None,
        username: str = None,
        image_url: str = None,
        role: str = None,
        **kwargs
    ):
        self.session_id = session_id
        self.user_id = user_id
        self.name = name
        self.username = username
        self.role = role
        self.image_url = image_url

    def from_json(self, data=None):
        if data is not None:
            self.session_id = data.get("id")
            self.name = data.get("name")
            self.username = data.get("username")
            self.role = data.get("role")
            self.image_url = data.get("imageUrl")
            self.user_id = data.get("userId")
        return self


class SessionInfo(SwitchObject):
    def __init__(
        self,
        app: "swibots.Client" = None,
        id: str = None,
        app_id: str = None,
        allow_members: bool = False,
        members: List[SessionUser] = None,
        chat_group_id: str = None,
        **kwargs
    ):
        self.id = id
        self.app_id = app_id
        self.allow_members = allow_members
        self.members = members
        self.chat_group_id = chat_group_id

    def from_json(self, data=None):
        if data is not None:
            self.id = data.get("id")
            self.app_id = data.get("appId")
            self.allow_members = data.get("canMemberControlScreen")
            self.members = [
                SessionUser().from_json(member) for member in data.get("members", [])
            ]
            self.chat_group_id = data.get("groupId")
        return self



from typing import Generic, TypeVar
from http.client import responses

from swibots.utils.types import RequestMethod


T = TypeVar("T")


class RestResponse(Generic[T]):
    def __init__(self, data: T, status_code: int, headers: dict):
        self.data = data
        self.status_code = status_code
        self.headers = headers

    @property
    def is_error(self) -> bool:
        return self.status_code >= 400

    @property
    def error_message(self) -> str:
        err = (
            self.data
            if self.data is not None and self.data != ""
            else responses[self.status_code]
        )
        return f"Error {self.status_code}: {err}"



import json
from typing import List, Tuple, Type, TypeVar
import swibots, httpx
from swibots.errors import (
    NetworkError,
    UnAuthorized,
    InvalidRouteCall,
    BadRequest,
    ServerError,
)
from swibots.utils import RestClient
from swibots.utils.types import JSONDict
from swibots.base import RestResponse
from logging import getLogger

T = TypeVar("T", bound="swibots.SwitchObject")
LOG = getLogger(__name__)


class SwitchRestClient(RestClient):
    def __init__(
        self, app: "swibots.Client" = None, base_url: str = None, token: str = None
    ):
        super().__init__()
        self._app = app
        self._auth_token = token
        self._base_url = base_url

    @property
    def app(self) -> "swibots.Client":
        return self._app

    @app.setter
    def app(self, app: "swibots.Client"):
        self._app = app

    @property
    def token(self):
        return self._app.token

    @property
    def user(self) -> "swibots.AuthUser":
        return self._app.user

    @property
    def base_url(self, url: str):
        self._base_url = url

    @base_url.setter
    def base_url(self, url: str):
        self._base_url = url

    def build_object(self, obj_type: Type[T], data: JSONDict) -> T:
        return obj_type.build_from_json(data, self.app)

    def build_list(self, obj_type: Type[T], data: JSONDict) -> List[T]:
        return obj_type.build_from_json_list(data, self.app)

    def sync_post(self, url: str, data: dict = None, headers: dict = None):
        request = httpx.post(
            self._base_url + url,
            data=data,
            headers=self.prepare_request_headers(headers),
        )
        return self.parse_response((request.status_code, request.content))

    def sync_get(self, url: str, headers: dict = None) -> RestResponse[JSONDict]:
        request = httpx.get(
            self._base_url + url, headers=self.prepare_request_headers(headers)
        )
        return self.parse_response((request.status_code, request.content))

    async def get(
        self, url: str, data: dict = None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.get`."""
        return await self.do_request(url, "GET", data, headers=headers)

    async def post(
        self,
        url: str,
        data: dict = None,
        form_data=None,
        files=None,
        headers: dict = None,
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.post`."""
        return await self.do_request(url, "POST", data, form_data, files, headers)

    async def put(
        self,
        url: str,
        data: dict = None,
        form_data=None,
        files=None,
        headers: dict = None,
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.put`."""
        return await self.do_request(url, "PUT", data, form_data, files, headers)

    async def delete(
        self, url: str, data: dict = None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.delete`."""
        return await self.do_request(url, "DELETE", data, headers=headers)

    async def patch(
        self,
        url: str,
        data: dict = None,
        form_data=None,
        files=None,
        headers: dict = None,
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.patch`."""
        return await self.do_request(url, "PATCH", data, form_data, files, headers)

    async def head(
        self, url: str, data: dict = None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.head`."""
        return await self.do_request(url, "HEAD", data, headers)

    async def options(
        self, url: str, data: dict = None, headers: dict = None
    ) -> RestResponse[JSONDict]:
        """See :meth:`BaseRequest.options`."""
        return await self.do_request(url, "OPTIONS", data, headers)

    def parse_response(self, response: Tuple[int, bytes]) -> RestResponse[JSONDict]:
        decoded_s = response[1].decode("utf-8", "replace")
        try:
            jsonObject = json.loads(decoded_s)
        except ValueError as exc:
            jsonObject = decoded_s

        response = RestResponse(jsonObject, response[0], {})
        if response.is_error:
            error = response.error_message
            if ":" in error:
                error = error.split(":", maxsplit=1)[-1].strip().replace("'", '"')
                try:
                    errorJson = json.loads(error)
                    error = errorJson["message"]
                    if errorJson.get("status") == "BAD_REQUEST":
                        raise BadRequest(error)
                except (KeyError, json.JSONDecodeError):
                    pass
            if response.status_code == 401:
                raise UnAuthorized(error)
            elif response.status_code == 404:
                raise InvalidRouteCall(error)
            elif response.status_code == 503:
                raise ServerError(error)
            raise NetworkError(error)
        return response

    def prepare_request_data(self, data: dict) -> dict:
        return super().prepare_request_data(data)

    def prepare_request_headers(self, headers: dict) -> dict:
        headers = super().prepare_request_headers(headers)
        if self.token is not None:
            headers["Authorization"] = f"Bearer {self.token}"
        return headers

    async def do_request(
        self,
        path: str,
        method: str,
        data: dict = None,
        form_data=None,
        files=None,
        headers: dict = None,
    ) -> RestResponse[JSONDict]:
        data = self.prepare_request_data(data)
        headers = self.prepare_request_headers(headers)
        return self.parse_response(
            await RestClient.do_request(
                self,
                self._base_url + (path if path is not None else ""),
                method,
                data,
                form_data,
                files,
                headers,
            )
        )



from typing import List
from .types import Component, Icon, Any, Dict


class ShortVideo(Component):
    type = "short_video"

    def __init__(
        self,
        url: str = None,
        icons: List[Icon] = None,
        user_image: str = None,
        user_name: str = None,
        title: str = None,
        description: str = None,
    ):
        self.url = url
        self.icons = icons
        self.user_image = user_image
        self.user_name = user_name
        self.title = title
        self.description = description

    def to_json(self):
        data = {
            "type": "short_video",
            "videoUrl": self.url,
            "bottom": [icon.to_json() for icon in self.icons],
            "extraOptions": {
                "userAvatar": self.user_image,
                "userName": self.user_name,
                "description": self.description,
                "title": self.title,
            },
        }
        return data


class FeedPanel(Component):
    def __init__(self, feeds, next_callback: str):
        self.feeds = feeds
        self.next_callback = next_callback

    def to_json(self) -> Dict[str, Any]:
        data = {
            "type": "feed_panel",
            "feeds": [feed.to_json() for feed in self.feeds],
            "offsetCallbackData": self.next_callback,
        }
        return data



import logging

logging.basicConfig(level=logging.INFO)

import asyncio
from glob import glob
from swibots import (
    BotApp,
    BotContext,
    CommandEvent,
    InlineMarkup,
    InlineKeyboardButton,
    MediaUploadRequest,
    RegisterCommand,
    CommandHandler,
    UploadProgress
)

app = BotApp(
    "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6OTc1LCJpc19ib3QiOnRydWUsImFjdGl2ZSI6dHJ1ZSwiaWF0IjoxNjg5MTU5NjEzLCJleHAiOjIzMjAzMTE2MTN9.lfLspX1PKUdOdKDPMFHe3PtGSycdUEnA95QbUahSB6k"
).register_command(
    [RegisterCommand("start", "Ok", False), RegisterCommand("handler", "ok")]
)

# app.update_bot_commands()

@app.on_command("start")
async def pas(ctx: BotContext[CommandEvent]):
    m = ctx.event.message
    app.register_command(
        [RegisterCommand("hiii", "d")]
    )
    return
    for _ in range(5):
        message = m._prepare_response()
        message.message = "Text"
        await m.reply(message, MediaUploadRequest(
            "../Switch.zip",   
            block=True
        ))
    print("no blocking")
    return
    ctx.bot.update_bot_commands()
    app.register_command(
        [
            RegisterCommand("standup", "lot see")
        ]
    )
    await ctx.update_bot_commands()
    await m.send("Hi")
    print(m)
    async def prog(ct: UploadProgress):
         print(ct.file_name, ct.current, ct.readed)
    return
    print(await asyncio.gather(*[
         message.send("hmm", MediaUploadRequest("file.zip", callback=prog, block=True))  for _ in range(10)
     ]))

    #    print(ctx.handlers)
#    for f in glob("swibots/api/*py"):
 #       await m.send("hi", MediaUploadRequest(f))
#     async def prog(ct: UploadProgress):
#         print(ct.file_name, ct.current, ct.readed)
#     print(await asyncio.gather(*[
#         m.send("hmm", MediaUploadRequest("../Switch.zip", callback=prog, block=True))  for _ in range(10)
#     ]))
# #        ]:
#  #       await _
#     #    async def mm(ct):
#     #       print(ct)
#     #  ctx.bot.add_handler(CommandHandler("handler", mm))
#     return
#     for file in glob("swibots/api/bot/***py"):
#         print(file)
#         await m.respond(media=MediaUploadRequest(file))
#     #        return
#     return
    c = await m.respond(
        "Hi", inline_markup=InlineMarkup([[InlineKeyboardButton("See", url="uo")]])
    )
    await asyncio.sleep(2)
    await c.edit_text(
        "Hello",
        inline_markup=InlineMarkup(
            [
                [InlineKeyboardButton("Koila", url="aa")],
                [InlineKeyboardButton("Hmm", url="aa")],
            ]
        ),
    )

app.register_command(
    [RegisterCommand("hello", "description")]
)

async def start_and_edit():
    await app.start()
    print(app._register_commands)
    print(await app.get_message(75555))

app.run()

#app._loop.run_until_complete(start_and_edit())
#app._loop.run_forever()


# app.run()



from .base_handler import BaseHandler
from .event_handler import EventHandler
from .command_handler import CommandHandler
from .message_handler import MessageHandler
from .callback_query_handler import CallbackQueryHandler
from .channel_created_handler import ChannelCreatedHandler
from .channel_deleted_handler import ChannelDeletedHandler
from .channel_updated_handler import ChannelUpdatedHandler
from .group_created_handler import GroupCreatedHandler
from .group_deleted_handler import GroupDeletedHandler
from .group_updated_handler import GroupUpdatedHandler
from .member_joined_handler import MemberJoinedHandler
from .member_left_handler import MemberLeftHandler
from .user_banned_handler import UserBannedHandler
from .community_updated_handler import CommunityUpdatedHandler
from .unknown_command_handler import UnknownCommandHandler
from .inline_query_handler import InlineQueryHandler


class Handlers(
    CommandHandler,
    MessageHandler,
    CallbackQueryHandler,
    UnknownCommandHandler,
    ChannelCreatedHandler,
    ChannelDeletedHandler,
    ChannelUpdatedHandler,
    GroupCreatedHandler,
    GroupDeletedHandler,
    GroupUpdatedHandler,
    MemberJoinedHandler,
    MemberLeftHandler,
    UserBannedHandler,
    CommunityUpdatedHandler,
    InlineQueryHandler,
):
    pass



from typing import Callable
import swibots
from swibots.bots.filters.filter import Filter


class OnChannelCreated:
    def on_channel_created(
        self: "swibots.Client" = None, filter: Filter = None
    ) -> Callable:
        """Decorator for handling channel creations."""

        def decorator(func: Callable) -> Callable:
            if isinstance(self, swibots.Client):
                self.add_handler(
                    swibots.bots.handlers.ChannelCreatedHandler(func, filter)
                )

            return func

        return decorator



from .utils import *
from .api import *
from .base import *
from .errors import *
from .config import *
from .types import *

# from .bot_client import SwitchBotClient
from .bot_app import BotApp, Client
from .bots import *

App = BotApp



from typing import Optional, Literal
import swibots
from swibots.api.common.events.event import Event
from swibots.api.community.models.channel import Channel
from swibots.api.community.models.community import Community
from swibots.api.community.models.group import Group
from swibots.api.common.models.user import User
from swibots.api.callback import AppPage
from swibots.api.callback import CallbackResponse
from swibots.api.chat.models.message import Message
from swibots.types import EventType
from swibots.utils.types import JSONDict
from .command_event import CommandEvent


class CallbackQueryEvent(CommandEvent):
    """Message event"""

    def __init__(
        self,
        app: "swibots.App" = None,
        type: Optional[EventType] = None,
        community_id: Optional[str] = None,
        community: Optional[Community] = None,
        group_id: Optional[str] = None,
        group: Optional[Group] = None,
        channel_id: Optional[str] = None,
        channel: Optional[Channel] = None,
        action_by_id: Optional[int] = None,
        action_by: Optional[User] = None,
        data: Optional[dict] = None,
        user_id: Optional[str] = None,
        user: Optional[User] = None,
        message: Optional[Message] = None,
        command: Optional[str] = None,
        params: Optional[str] = None,
        callback_data: Optional[JSONDict] = None,
        query_id: Optional[str] = None,
        details: Optional[CallbackResponse] = None,
        app_session_id: Optional[str] = None,
    ):
        super().__init__(
            app=app,
            type=type or EventType.CALLBACK_QUERY,
            community_id=community_id,
            community=community,
            group_id=group_id,
            group=group,
            channel_id=channel_id,
            channel=channel,
            action_by_id=action_by_id,
            action_by=action_by,
            data=data,
            user_id=user_id,
            user=user,
            message=message,
            command=command,
            params=params,
        )
        self.callback_data = callback_data
        self.query_id = query_id
        self.details = details
        self.app_session_id = app_session_id

    def from_json(self, data: JSONDict) -> "CallbackQueryEvent":
        super().from_json(data)
        if data is not None:
            self.callback_data = self.data.get("callbackData")
            self.query_id = self.data.get("callbackQueryId")
            self.details = CallbackResponse().from_json(
                self.data.get("message", {}).get("additionalDetails")
            )
            self.app_session_id = self.data.get("message", {}).get("appSessionId")
        return self

    async def __action_callback(self, **kwargs):
        resp = await self.app.bots_service.post(
            "/v1/bots/callback/answer",
            data={
                "type": "action_callback",
                "callbackQueryId": self.query_id,
                "messageId": str(self.message.id),
                **kwargs,
            },
        )
        return resp.data

    async def share(self, text: str):
        """Prompt a user to share text"""
        return await self.__action_callback(action="share", url=text)

    async def copy(self, text: str):
        """Copy text to user's clipboard.

        text (str): text which should be copied to user's device."""
        return await self.__action_callback(action="copy", url=text)

    async def redirect(self, url: str):
        """Redirect user to external url.

        url (str): url to open"""
        return await self.__action_callback(action="navigate", url=url)

    async def show_ad(
        self,
        id: str,
        ad_type: Literal["VIDEO_1", "VIDEO_2"],
        success_callback: str,
    ):
        """Display ad to the user on any callback action.

        id: session id to join.
        ad_type: either 'VIDEO_1' or 'VIDEO_2'
        success_callback: Callback to trigger after successfully showing.
        """
        return await self.__action_callback(
            action="show_ad",
            addId=id,
            addType=ad_type,
            extraOptions={"adCallback": success_callback},
        )

    async def request_session_join(self, session_id: str):
        """Request user to join the session, from session id

        session_id: (str)
        """
        return await self.__action_callback(url=session_id, action="joinSession")

    async def request_session_create(self, callback_data: str = None):
        """Create the bot session on the behalf of user.

        callback_data: str: Callback data to trigger after creating"""
        return await self.__action_callback(url=callback_data, action="createSession")

    async def answer(
        self,
        text: str = None,
        url: Optional[str] = None,
        show_alert: Optional[bool] = False,
        cache_time: Optional[int] = None,
        callback: Optional[AppPage] = None,
        new_page: bool = False,
    ) -> bool:
        """Answer callback query"""
        if callback:
            query_id = self.query_id
            if not new_page and self.details.parent_id:
                query_id = self.details.parent_id
            return await self.app.answer_ui_query(
                query_id,
                callback=callback,
                message_id=self.message.id,
                app_session_id=self.app_session_id,
            )
        return await self.app.answer_callback_query(
            self.query_id,
            message_id=self.message.id,
            text=text,
            url=url,
            show_alert=show_alert,
            cache_time=cache_time,
            app_session_id=self.app_session_id,
        )



from .Accordian import Accordian
from .AppPage import AppPage, AppBar
from .Dropdown import Dropdown
from .ListView import ListView, ListTile, ListViewType, ListTileProgress
from .ListItem import ListItem
from .Grid import Grid, GridItem, GridType
from .Search import SearchBar, SearchHolder
from .Progress import Progress, ProgressStyle, Size
from .Tab import TabBar, TabBarTile, TabBarType
from .types import Text, Image, Icon, ScreenType, TextSize, Expansion, Spacer, Badge
from .BottomBar import BottomBarTile, BottomBarType, BottomBar
from .Players import VideoPlayer, Embed, AudioPlayer, FileViewer
from .Inputs import TextInput, FilePicker
from .Carousel import Carousel
from .FAB import FAB
from .Feed import FeedPanel, ShortVideo
from .Card import Card, CardView, CardSize, CardStyle, Banner
from .Button import (
    Button,
    AdButton,
    ButtonGroup,
    DownloadButton,
    StickyHeader,
    ClipboardButton,
    ShareButton,
    ButtonVariant,
)
from .callbackResponse import CallbackResponse
from .Table import Table, TableTile

__all__ = [
    "Accordian",
    "AdButton",
    "AppBar",
    "AppPage",
    "AudioPlayer",
    "Badge",
    "Banner",
    "BottomBar",
    "BottomBarTile",
    "BottomBarType",
    "Button",
    "ButtonGroup",
    "ButtonVariant",
    "Card",
    "CardView",
    "CardSize",
    "CardStyle",
    "Carousel",
    "ClipboardButton",
    "DownloadButton",
    "Dropdown",
    "Embed",
    "Expansion",
    "FAB",
    "FeedPanel",
    "FilePicker",
    "FileViewer",
    "Grid",
    "GridItem",
    "GridType",
    "Icon",
    "Image",
    "ListItem",
    "ListTile",
    "ListTileProgress",
    "ListView",
    "ListViewType",
    "Progress",
    "ProgressStyle",
    "ScreenType",
    "SearchBar",
    "SearchHolder",
    "ShareButton",
    "ShortVideo",
    "Size",
    "Spacer",
    "StickyHeader",
    "Table",
    "TableTile",
    "TabBar",
    "TabBarTile",
    "TabBarType",
    "Text",
    "TextInput",
    "TextSize",
    "VideoPlayer",
]



from .community_controller import *
from .channel_controller import *
from .group_controller import *
from .roles_controller import *
from .permissions_controller import *
from .rolemember_controller import *
from .restrict_controller import *
from .ban_controller import *
from .quest_controller import *
from .messaging_controller import InstantMessagingController



import swibots
from enum import Enum
from swibots.base import SwitchObject
from typing import Literal

ADS_OPTIONS = ["VIDEO_1", "VIDEO_2"]


class ADInfo(SwitchObject):
    def __init__(
        self,
        app: "swibots.Client" = None,
        access_id: str = None,
        admin_id: str = None,
        app_id: str = None,
        id: str = None,
        type: Literal["VIDEO_1", "VIDEO_2", "IMAGE"] = None,
        **kwargs
    ):
        super().__init__(app, **kwargs)
        self.type = type
        self.app_id = app_id
        self.id = id
        self.admin_id = admin_id
        self.access_id = access_id

    def to_json(self):
        return {
            "id": self.id,
            "appId": self.app_id,
            "adminId": self.admin_id,
            "addAccessId": self.access_id,
            "type": self.type,
        }



import logging
from typing import TYPE_CHECKING, List
from datetime import timedelta, datetime
from swibots.api.community.models import RestrictedUser

if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community/restrict"


class RestrictController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def list_restricted_users(self, community_id: str) -> List[RestrictedUser]:
        response = await self.client.get(
            f"{BASE_PATH}/users?communityId={community_id}"
        )
        return self.client.build_list(RestrictedUser, response.data)

    async def get_restricted_user(self, community_id: str, user_id: int):
        response = await self.client.get(
            f"{BASE_PATH}/user?communityId={community_id}&userId={user_id}"
        )
        return self.client.build_object(RestrictedUser, response.data.get("result"))

    async def restrict_user(
        self,
        community_id: str,
        restricted: bool,
        user_id: int,
        until_date: int = 0,
    ) -> bool:
        await self.client.post(
            f"{BASE_PATH}/user",
            data={
                "communityId": community_id,
                "restricted": restricted,
                "restrictedTillTimestamp": until_date,
                "userId": user_id,
            },
        )
        return True

    async def update_restricted_user(
        self,
        community_id: str,
        restricted: bool,
        user_id: int,
        until_date: int = 0,
    ) -> bool:
        await self.client.put(
            f"{BASE_PATH}/user",
            data={
                "communityId": community_id,
                "restricted": restricted,
                "restrictedTillTimestamp": until_date,
                "userId": user_id,
            },
        )
        return True



from .rest_client import RestClient
from .ws import *
from .types import *
from urllib.parse import urlparse


def isUrl(text):
    """Validate, if text is url."""
    parse = urlparse(text)
    return parse.netloc and parse.scheme



import re
from typing import TYPE_CHECKING, Optional, TypeVar
from swibots.bots.filters.filter import Filter
from swibots.types import EventType

from swibots.utils.types import SCT, HandlerCallback
from swibots.bots.handlers.event_handler import EventHandler
from swibots.bots.bot_context import BotContext
from swibots.api.chat.events import CommandEvent

VALID_COMMAND_REGEX = r"^[\da-z0-9_]{1,32}$"

if TYPE_CHECKING:
    pass

ResType = TypeVar("ResType")


class CommandHandler(EventHandler):
    def __init__(
        self,
        command: SCT[str],
        callback: HandlerCallback[BotContext[CommandEvent], ResType],
        filter: Optional[Filter] = None,
        **kwargs,
    ):
        super().__init__(EventType.COMMAND, callback, filter, **kwargs)
        if isinstance(command, str):
            commands = frozenset({command.lower()})
        else:
            commands = frozenset(x.lower() for x in command)
        for comm in commands:
            if not re.match(VALID_COMMAND_REGEX, comm):
                raise ValueError(f"Command `{comm}` is not a valid bot command.")
        self.commands = commands

    async def should_handle(self, context: BotContext[CommandEvent]) -> bool:
        return await super().should_handle(
            context,
            precheck=lambda context: (
                context.event.command is not None
                and context.event.command in self.commands
                and context.event.message.user_id != context.app.user.id
                and context.event.message is not None
            ),
        )



from swibots.utils.types import JSONDict
from .types import Component
from typing import List, Optional, Any, Dict
from .ListItem import ListItem


class Dropdown(Component):
    type = "dropdown"

    def __init__(
        self,
        placeholder: str = None,
        selected: int = None,
        options: List[ListItem] = None,
        disabled: bool = None,
        max_size: bool = None,
    ):
        self.disabled = disabled
        self.options = options
        self.selected = selected
        self.placeholder = placeholder
        self.max_size = max_size

    def to_json(self) -> JSONDict:
        data = {
            "type": self.type,
            "placeHolder": self.placeholder,
            "options": [listitem.to_json() for listitem in self.options],
            "disabled": self.disabled,
        }
        if self.selected:
            data["selectedIndex"] = self.selected
        data["mainAxisSize"] = "max" if self.max_size else "min"
        return data



from typing import Type, TypeVar
import swibots
from swibots.api.auth.models import AuthUser, AuthResult
from swibots.base import SwitchRestClient
from swibots.config import get_config


T = TypeVar("T", bound="swibots.AuthUser")


class AuthClient(SwitchRestClient):
    """Auth client

    This client is used to communicate with the auth service.

    Controllers:
        - :attr:`users`: :obj:`~switch.api.auth.controllers.UserController` : The users controller
    """

    def __init__(self, app: "swibots.App", base_url: str = None):
        """Initialize the auth client"""
        base_url = base_url or get_config()["AUTH_SERVICE"]["BASE_URL"]
        super().__init__(app, base_url)
        self._authorization = None

    def prepare_request_headers(self, headers: dict) -> dict:
        headers = super().prepare_request_headers(headers)
        if self.token is not None:
            headers["authtoken"] = f"{self.token}"
        return headers

    def get_me_sync(self, user_type: Type[T] = AuthUser) -> T:
        user_info = self.sync_get("/api/user").data
        return self.build_object(user_type, user_info)

    async def get_me(self, user_type: Type[T] = AuthUser) -> T:
        """Get the current user

        Parameters:
            user_type (``Type[T]``, *optional*): The user type to return. Defaults to :obj:`~switch.api.auth.models.AuthUser`.

        Returns:
            ``T``: The current user

        This functions does the same as :meth:`~switch.api.auth.controllers.UserController.me`.

        """
        response = await self.get("/api/user")
        return self.build_object(user_type, response.data)

    def login(self, email: str, password: str):
        response = self.sync_post(
            "/api/login", data={"email": email, "password": password}
        )
        return self.build_object(AuthResult, response.data)



import os, pickle, time, aiofiles


class AsyncRateLimiter:
    def __init__(self, storage_file="swibots.pkl"):
        self.storage_file = storage_file
        self.rate_limit_data = {}

    async def _load_data(self):
        if os.path.exists(self.storage_file):
            async with aiofiles.open(self.storage_file, "rb") as f:
                content = await f.read()
                self.rate_limit_data = pickle.loads(content)

    async def _save_data(self):
        async with aiofiles.open(self.storage_file, "wb") as f:
            await f.write(pickle.dumps(self.rate_limit_data))

    def limit(self, key: str, times: int, seconds: int):
        def decorator(func):
            async def async_wrapper(*args, **kwargs):
                current_time = time.time()

                await self._load_data()

                if key not in self.rate_limit_data:
                    self.rate_limit_data[key] = []

                self.rate_limit_data[key] = [
                    t for t in self.rate_limit_data[key] if current_time - t < seconds
                ]

                if len(self.rate_limit_data[key]) >= times:
                    raise RateLimitException(
                        f"Rate limit exceeded: {times} calls in {seconds} seconds"
                    )

                # Record the call
                self.rate_limit_data[key].append(current_time)
                await self._save_data()
                return await func(*args, **kwargs)

            return async_wrapper

        return decorator


class RateLimitException(Exception):
    pass



from pyrogram.client import Client
from pyrogram import filters
from datetime import datetime

app = Client(
    "bot",
    api_id=6,
    api_hash="eb06d4abfb49dc3eeb1aeb98ae0f581e",
    workers=100,
    bot_token="6248620141:AAFvCZVXdbe9OPS0R2Yc-9xfnMAnsM9VL4E"
)

@app.on_message(filters.document)
async def download(client, message):
    st = datetime.now()
    await message.download()
    print((datetime.now() - st).total_seconds() / 60)
app.run()


import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from swibots.api.chat import ChatClient

from swibots.api.chat.models import Organization, OrgApp
from swibots.api.common import User
from urllib.parse import urlencode

log = logging.getLogger(__name__)

BASE_PATH = "/v1/organization"


class OrganizationController:
    """Organization controller"""

    def __init__(self, client: "ChatClient") -> None:
        self.client = client

    async def get_organizations(
        self, bot_id: int = None, community_id: str = None, user_id: str = None
    ):
        param = urlencode(
            {"botId": bot_id, "communityId": community_id, "userId": user_id}
        )
        response = await self.client.get(f"{BASE_PATH}/get-organizations?{param}")
        return self.client.build_list(Organization, response.data)

    async def get_organization_by_id(self, id: str):
        response = await self.client.get(f"{BASE_PATH}/{id}")
        return self.client.build_object(Organization, response.data)

    async def get_organization_apps(self, id: str):
        response = await self.client.get(f"{BASE_PATH}/apps?orgId={id}")
        return self.client.build_list(OrgApp, response.data)

    async def get_organization_followers(self, id: str):
        response = await self.client.get(f"{BASE_PATH}/followers?orgId={id}")
        return self.client.build_list(User, response.data)



import enum

MAX_FILE_SIZE = 8589934592


class EventType(enum.Enum):
    """Represents the type of a event."""

    # Chat
    MESSAGE = "MESSAGE"
    COMMAND = "COMMAND"
    CALLBACK_QUERY = "CALLBACK_QUERY"
    INLINE_QUERY = "INLINE_QUERY"

    # Community
    COMMUNITY_CHANNEL_CREATE = "COMMUNITY_CHANNEL_CREATE"
    COMMUNITY_CHANNEL_UPDATE = "COMMUNITY_CHANNEL_UPDATE"
    COMMUNITY_CHANNEL_DELETE = "COMMUNITY_CHANNEL_DELETE"
    COMMUNITY_GROUP_CREATE = "COMMUNITY_GROUP_CREATE"
    COMMUNITY_GROUP_UPDATE = "COMMUNITY_GROUP_UPDATE"
    COMMUNITY_GROUP_DELETE = "COMMUNITY_GROUP_DELETE"
    COMMUNITY_USER_BAN = "COMMUNITY_USER_BAN"
    COMMUNITY_USER_UNBAN = "COMMUNITY_USER_UNBAN"
    COMMUNITY_MEMBER_JOIN = "COMMUNITY_MEMBER_JOIN"
    COMMUNITY_MEMBER_LEAVE = "COMMUNITY_MEMBER_LEAVE"
    COMMUNITY_UPDATE = "COMMUNITY_UPDATE"
    COMMUNITY_DELETE = "COMMUNITY_DELETE"

    COMMUNITY_UNRESTRICT_USER = "COMMUNITY_UN_RESTRICT_USER"
    COMMUNITY_RESTRICT_USER = "COMMUNITY_RESTRICT_USER"


class MediaType(enum.Enum):
    IMAGE = 1
    VIDEO = 2
    AUDIO = 3
    DOCUMENT = 7
    STICKER = 200


def GetMediaType(media_type: int):
    try:
        return {
            1: "IMAGE",
            2: "VIDEO",
            3: "AUDIO",
            4: "EMBED",
            7: "FILE",
            200: "STATIC_STICKER",
            201: "ANIMATED_STICKER",
            202: "VIDEO_STICKER",
        }[media_type]
    except KeyError:
        return None



import asyncio
import os
import swibots

from swibots.utils.types import DownloadProgressCallback
from swibots.errors import SwitchError, MediaEmpty
from swibots.api.chat.models import Message
from typing import Union, Optional, Callable, BinaryIO

DOWNLOAD_MEDIA = "downloads/"


class DownloadMedia:
    async def download_media(
        self: "swibots.ApiClient",
        message: Message,
        file_name: str = DOWNLOAD_MEDIA,
        in_memory: bool = False,
        block: bool = True,
        progress: DownloadProgressCallback = None,
        progress_args: tuple = (),
    ) -> Optional[Union[BinaryIO, bytes, str]]:
        """
        Download media by message reference
        """
        if not message.is_media:
            raise MediaEmpty("Message is not a media message")

        if message.media_link is None:
            raise MediaEmpty("Message does not have a media link")

        media_file_name = message.media_link.split("/")[-1]
        if file_name and os.path.isdir(file_name):
            directory = file_name
            file_name = media_file_name
        else:
            directory, file_name = os.path.split(file_name or "")
            file_name = file_name or media_file_name or ""

        download_fn = self.handle_download(
            url=message.media_link,
            directory=directory,
            file_name=file_name,
            in_memory=in_memory,
            progress=progress,
            progress_args=progress_args,
        )
        if block:
            return await download_fn
        return asyncio.get_event_loop().create_task(download_fn)



from typing import Optional
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject
import swibots


class RolePermission(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        id: Optional[int] = None,
        add_members: Optional[bool] = None,
        add_roles: Optional[bool] = None,
        send_messages: Optional[bool] = None,
        ban_users: Optional[bool] = None,
        change_info: Optional[bool] = None,
        delete_messages: Optional[bool] = None,
        dm_permission: Optional[bool] = None,
        pin_messages: Optional[bool] = None,
        restrict_messaging: Optional[bool] = None,
        can_deduct_xp: Optional[bool] = None,
        role_id: Optional[int] = None,
        created_at: Optional[str] = None,
        updated_at: Optional[str] = None,
    ):
        super().__init__(app)
        self.id = id
        self.add_members = add_members
        self.add_roles = add_roles
        self.send_messages = send_messages
        self.ban_users = ban_users
        self.change_info = change_info
        self.delete_messages = delete_messages
        self.dm_permission = dm_permission
        self.pin_messages = pin_messages
        self.restrict_messaging = restrict_messaging
        self.can_deduct_xp = can_deduct_xp
        self.role_id = role_id
        self.created_at = created_at
        self.updated_at = updated_at

    def to_json(self) -> JSONDict:
        return {
            "id": self.id,
            "roleId": self.role_id,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "rolePermission": {
                "addNewMembers": self.add_members,
                "addNewRoles": self.add_roles,
                "allowedToSendMessageInChannels": self.send_messages,
                "banUsers": self.ban_users,
                "changeCommunityInfo": self.change_info,
                "deletePostsAndMessages": self.delete_messages,
                "hasDMPermission": self.delete_messages,
                "pinMessages": self.pin_messages,
                "canDeductXPFromUser": self.can_deduct_xp,
                "restrictMessaging": self.restrict_messaging,
            },
        }

    def from_json(self, data: JSONDict) -> "RolePermission":
        if data is not None:
            self.created_at = data.get("createdAt")
            self.updated_at = data.get("updatedAt")

            if data.get("rolePermission"):
                data = data["rolePermission"]

            self.add_members = data.get("addNewMembers")
            self.send_messages = data.get("allowedToSendMessageInChannels")
            self.delete_messages = data.get("deletePostsAndMessages")
            self.pin_messages = data.get("pinMessages")
            self.change_info = data.get("changeCommunityInfo")
            self.add_roles = data.get("addNewRoles")
            self.can_deduct_xp = data.get("canDeductXPFromUser")
            self.ban_users = data.get("banUsers")
            self.dm_permission = data.get("hasDMPermission")
            self.restrict_messaging = data.get("restrictMessaging")
        return self



import logging
from typing import TYPE_CHECKING

from swibots.api.community.models import Role, RolePermission


if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community/permission"


class PermissionController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def get_permission(self, role_id: str) -> RolePermission:
        """Get permission by role id"""
        response = await self.client.get(f"{BASE_PATH}/get/?roleId={role_id}")
        return self.client.build_object(RolePermission, response.data.get("result"))

    async def add_permission(self, permission: RolePermission) -> bool:
        """Add permission"""
        response = await self.client.post(
            f"{BASE_PATH}/add", data=permission.to_json_request()
        )
        return response.data.get("success", False)

    async def update_permission(
        self,
        permission: RolePermission = None,
        permission_id: int = 0,
        role_id: int = 0,
    ) -> bool:
        """update permission"""
        if id:
            permission.id = permission_id
        if role_id:
            permission.role_id = role_id

        await self.client.put(
            f"{BASE_PATH}/update",
            data=permission.to_json_request(),
        )
        return True

    async def delete_permission(self, permission_id: int) -> bool:
        """delete permission by id"""
        response = await self.client.delete(f"{BASE_PATH}/delete/{permission_id}")
        return response.data.get("success", False)

    async def add_user_permission(
        self,
        community_id: str,
        user_id: int,
        role_colour: str,
        role_name: str,
        permission: RolePermission,
    ) -> bool:
        resp = await self.client.post(
            f"{BASE_PATH}/user",
            data={
                "communityId": community_id,
                "memberId": user_id,
                "roleColour": role_colour,
                "roleName": role_name,
                "rolePermission": permission.to_json()["rolePermission"],
            },
        )
        return resp.data.get("success", False)



from typing import Type, TypeVar
import swibots
from swibots.api.chat.models import Message


class ClearConversation:
    async def clear_conversation(self: "swibots.ApiClient", receiver_id: int) -> bool:
        """Clear a conversation

        Parameters:
            receiver_id (``int``): The receiver id

        Returns:
            ``bool``: True if the conversation was cleared

        Raises:
            ``~switch.error.SwitchError``: If the conversation could not be cleared

        This method does the same as :meth:`~switch.api.chat.controllers.MessageController.clear_conversation`.
        """
        return await self.chat_service.messages.clear_conversation(receiver_id)



import json
import logging
from typing import TYPE_CHECKING, List, Optional
from swibots.api.bot.models import GameInfo

if TYPE_CHECKING:
    from swibots.api.bot import BotClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/bots/game"


class GameController:
    def __init__(self, client: "BotClient"):
        self.client = client

    async def create_leaderboard(
        self,
        user_id: int,
        score: int = 0,
        level: int = None,
        community_id: str = None,
        bot_id: int = None,
    ) -> GameInfo:
        if not bot_id:
            bot_id = self.client.user.id
        response = await self.client.post(
            f"{BASE_PATH}/leaderboard",
            data={
                "botId": bot_id,
                "userId": user_id,
                "score": score,
                "level": level,
                "communityId": community_id,
            },
        )
        return self.client.build_object(GameInfo, response.data)

    async def update_leaderboard(
        self,
        id: str,
        user_id: int,
        score: int = 0,
        level: int = 0,
        community_id: str = None,
        bot_id: int = None,
    ) -> None:
        if not bot_id:
            bot_id = self.client.user.id
        response = await self.client.put(
            f"{BASE_PATH}/leaderboard",
            data={
                "botId": bot_id,
                "userId": user_id,
                "score": score,
                "id": id,
                "level": level,
                "communityId": community_id,
            },
        )
        return self.client.build_object(GameInfo, response.data)

    async def get_global_leaderboard(
        self, bot_id: Optional[int] = None
    ) -> List[GameInfo]:
        if not bot_id:
            bot_id = self.client.user.id
        response = await self.client.get(
            f"{BASE_PATH}/leaderboard/global?botId={bot_id}"
        )
        return self.client.build_list(GameInfo, response.data)

    async def get_community_leaderboard(
        self, community_id: str, bot_id: Optional[int] = None
    ) -> List[GameInfo]:
        if not bot_id:
            bot_id = self.client.user.id
        response = await self.client.get(
            f"{BASE_PATH}/leaderboard/community?communityId={community_id}&botId={bot_id}"
        )
        return self.client.build_list(GameInfo, response.data)

    async def get_game_score(
        self,
        user_id: int,
        community_id: Optional[str] = None,
        bot_id: Optional[int] = None,
    ) -> GameInfo:
        if not bot_id:
            bot_id = self.client.user.id
        query = f"?botId={bot_id}&userId={user_id}"
        if community_id:
            query += f"&communityId={community_id}"
        response = await self.client.get(f"{BASE_PATH}/score{query}")
        return self.client.build_object(GameInfo, response.data)



import swibots
from swibots.api.community.models import BanInfo


class BanUser:
    async def ban_user(
        self: "swibots.ApiClient", community_id: str, user_id: str
    ) -> BanInfo:
        """
        Bans a user in a community.

        Args:
            community_id (str): The ID of the community.
            user_id (str): The ID of the user to ban.
        """
        return await self.community_service.ban.ban_user(community_id, user_id)



from .message import Message
from .inline_markup import InlineMarkup
from .inline_keyboard_button import InlineKeyboardButton
from .group_chat_history import GroupChatHistory
from .inline import *
from .sticker import Sticker, StickerPack
from .organization import Organization, OrgApp
from .session import SessionInfo, SessionUser
from .ads import ADInfo



from typing import Type, TypeVar
import swibots
from swibots.api.bot.models import BotInfo


class GetBotInfo:
    async def get_bot_info(self: "swibots.ApiClient", bot_id: str) -> BotInfo:
        """Get bot info

        Parameters:
            bot_id (``str``): The bot id. Defaults to the current bot id.

        Returns:
            :obj:``~switch.api.bot.models.BotInfo``: The bot info

        This functions does the same as :meth:`~switch.api.bot.controllers.BotController.get_bot_info`.
        """
        return await self.bots_service.bots.get_bot_info(bot_id=bot_id)



import swibots

from swibots.api.community.models import RolePermission


class PermissionMethods:
    """
    Methods for managing permissions in a community.

    Args:
        client (swibots.ApiClient): The API client.
    """

    async def get_permission(
        self: "swibots.ApiClient", role_id: str
    ) -> RolePermission | None:
        """
        Get the permission with the specified role ID.

        Args:
            role_id: The ID of the permission.

        Returns:
            The permission with the specified role ID, or `None` if the permission does not exist.
        """

        return await self.community_service.permission.get_permission(role_id)

    async def add_permission(
        self: "swibots.ApiClient", permission: RolePermission
    ) -> bool:
        """
        Add a new permission.

        Args:
            permission: The permission to add.

        Returns:
            True if the permission was added successfully, False otherwise.
        """

        return await self.community_service.permission.add_permission(permission)

    async def update_permission(
        self: "swibots.ApiClient",
        permission: RolePermission,
        permission_id: int = 0,
        role_id: int = 0,
    ) -> bool:
        """
        Update an existing permission.

        Args:
            permission: The permission to update.
            permission_id: The ID of the permission to update.
            role_id: The role ID of the permission to update.

        Returns:
            True if the permission was updated successfully, False otherwise.
        """

        return await self.community_service.permission.update_permission(
            permission, permission_id, role_id
        )

    async def delete_permission(self: "swibots.ApiClient", permission_id: int) -> bool:
        """
        Delete a permission.

        Args:
            permission_id: The ID of the permission to delete.

        Returns:
            True if the permission was deleted successfully, False otherwise.
        """

        return await self.community_service.permission.delete_permission(permission_id)

    async def add_user_permission(
        self: "swibots.ApiClient",
        community_id: str,
        user_id: int,
        role_colour: str,
        role_name: str,
        permission: RolePermission,
    ) -> bool:
        """
        Add a permission to a user.

        Args:
            community_id: The ID of the community.
            user_id: The ID of the user.
            role_colour: The role color of the permission.
            role_name: The role name of the permission.
            permission: The permission to add.

        Returns:
            True if the permission was added successfully, False otherwise.
        """

        return await self.community_service.permission.add_user_permission(
            community_id, user_id, role_colour, role_name, permission
        )



from typing import List, Optional
import swibots
from swibots.api.airdrop.models import Tournament


class GetTournaments:
    async def get_tournaments(
        self: "swibots.ApiClient", community_id: str
    ) -> List[Tournament]:
        """Get tournaments from community_id

        Args:
            community_id: community id to get tournaments from.

        Returns:
            A list of tournaments
        """
        return await self.airdrop_service.tournament.get_tournaments(community_id)



import asyncio
import os
import json
import logging
from typing import TYPE_CHECKING, List, Optional, Tuple, Union, Literal
from urllib.parse import urlencode
from io import BytesIO
from asyncio.tasks import Task
from swibots.types import MediaType
from swibots.api.chat.models import (
    Message,
    GroupChatHistory,
    InlineMarkup,
    InlineQuery,
    InlineQueryAnswer,
)
from swibots.utils import isUrl
from swibots.api.common.models import User, EmbeddedMedia, Media
from swibots.api.community.models import (
    Channel,
    Group,
    CommunityMember,
    SearchResultUser,
)

if TYPE_CHECKING:
    from swibots.api.chat import ChatClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/message"


class MessageController:
    """Message controller

    This controller is used to communicate with the message endpoints.

    """

    def __init__(self, client: "ChatClient"):
        self.client = client

    async def new_message(
        self,
        to: Optional[int | User] = None,
        channel: Optional[Channel | str] = None,
        group: Optional[Group | str] = None,
    ) -> Message:
        """Create a new message"""
        if isinstance(to, User):
            to = to.id

        if isinstance(channel, Channel):
            channel = channel.id

        if isinstance(group, Group):
            group = group.id

        return Message(
            user_id=self.client.user.id,
            receiver_id=to,
            channel_id=channel,
            group_id=group,
            app=self.client.app,
        )

    async def get_messages(
        self, user_id: int = None, limit: int = 100, offset: int = 0
    ) -> List[Message]:
        """Get messages for a user

        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved
        """
        if user_id is None:
            user_id = self.client.user.id
        data = {"limit": limit, "offset": offset}
        log.debug("Getting messages for user %s", user_id)
        response = await self.client.get(
            f"{BASE_PATH}/personal/{user_id}?{urlencode(data)}"
        )
        return self.client.build_list(Message, response.data)

    async def send_message(
        self,
        message: str,
        community_id: str = None,
        channel_id: str = None,
        group_id: str = None,
        user_id: Optional[int] = None,
        user_session_id: Optional[str] = None,
        document: Optional[str | BytesIO] = None,
        caption: Optional[str] = None,
        description: Optional[str] = None,
        embed_message: Optional[EmbeddedMedia] = None,
        inline_markup: Optional[InlineMarkup] = None,
        reply_to_message_id: Optional[int] = None,
        scheduled_at: Optional[int] = None,
        **kwargs,
    ) -> Message | Task:
        if not (user_id or (community_id and (group_id or channel_id))):
            raise ValueError(
                "No chat parameter provided, Either use user_id or group_id/channel_id with community_id."
            )
        if document:
            return await self.send_media(
                document=document,
                message=message,
                caption=caption,
                description=description,
                community_id=community_id,
                group_id=group_id,
                channel_id=channel_id,
                user_session_id=user_session_id,
                inline_markup=inline_markup,
                user_id=user_id,
                scheduled_at=scheduled_at,
                **kwargs,
            )

        new_message = Message(
            app=self.client.app,
            message=message,
            community_id=community_id,
            group_id=group_id,
            channel_id=channel_id,
            receiver_id=user_id,
            user_session_id=user_session_id,
            embed_message=embed_message,
            inline_markup=inline_markup,
            replied_to_id=reply_to_message_id,
            scheduled_at=scheduled_at,
            **kwargs,
        )

        if new_message.embed_message:
            if new_message.embed_message.thumbnail:
                thumb = new_message.embed_message.thumbnail
                if thumb and not isUrl(thumb):
                    new_message.embed_message.thumbnail = (
                        await self.client.app.upload_media(thumb)
                    ).url

        data = new_message.to_json_request()
        log.debug("Sending message %s", json.dumps(data))
        response = await self.client.post(f"{BASE_PATH}/create", data=data)
        return self.client.build_object(Message, response.data["message"])

    async def send_media(
        self,
        document: str = None,
        media: Optional[Media] = None,
        message: Optional[str] = "",
        community_id: Optional[str] = None,
        group_id: Optional[str] = None,
        channel_id: Optional[str] = None,
        user_id: Optional[int] = None,
        user_session_id: Optional[str] = None,
        caption: Optional[str] = None,
        description: Optional[str] = None,
        file_name: Optional[str] = None,
        mime_type: Optional[str] = None,
        media_type: Optional[int] = None,
        thumb: Optional[str | BytesIO] = None,
        blocking: Optional[bool] = True,
        progress: Optional[callable] = None,
        progress_args: Optional[tuple] = (),
        reply_to_message_id: Optional[int] = None,
        scheduled_at: Optional[int] = None,
        inline_markup: Optional[InlineMarkup] = None,
        part_size: Optional[int] = None,
        task_count: Optional[int] = None,
        premium: Optional[bool] = False,
        **kwargs,
    ) -> Message | Task:
        async def _upload_media(media):
            if document:
                force_storage_method = kwargs.get("secondary_upload", False)
                private_community = bool(user_id)
                if community_id:
                    community = await self.client.app.get_community(community_id)
                    private_community = not community.is_public

                media = await self.client.app.upload_media(
                    path=document,
                    caption=caption,
                    description=(
                        file_name or description or os.path.basename(document)
                        if isinstance(document, str)
                        else document.name
                    ),
                    mime_type=mime_type,
                    media_type=media_type,
                    callback=progress,
                    callback_args=progress_args,
                    thumb=thumb,
                    part_size=part_size,
                    task_count=task_count,
                    premium=premium,
                    for_document=kwargs.get("is_document")
                    or (kwargs.get("media_type", 0) == MediaType.DOCUMENT.value),
                    private_community=private_community or force_storage_method,
                )
            elif not media:
                raise ValueError("'media' or 'document' must be provided!")
            new_message = Message(
                app=self.client.app,
                receiver_id=user_id,
                community_id=community_id,
                group_id=group_id,
                channel_id=channel_id,
                user_session_id=user_session_id,
                media_info=media,
                replied_to_id=reply_to_message_id,
                message=message,
                inline_markup=inline_markup,
                media_link=media.url if media else None,
                scheduled_at=scheduled_at,
                **kwargs,
            )
            form = new_message.to_json_request()
            log.debug("Sending message %s", json.dumps(form))
            request_url = f"{BASE_PATH}/create"
            response = await self.client.post(request_url, data=form)
            return self.client.build_object(Message, response.data["message"])

        task = asyncio.create_task(_upload_media(media))
        if blocking:
            return await task
        return task

    async def edit_media(
        self,
        message_id: int,
        media_id: Optional[int] = None,
        message: Optional[str] = None,
        document: Optional[str] = None,
        thumb: Optional[str] = None,
        inline_markup: InlineMarkup = None,
        progress=None,
        progress_args: Optional[Tuple] = None,
        mime_type: Optional[str] = None,
        file_name: Optional[str] = None,
        **kwargs,
    ) -> Union[Message, Media]:
        msg = None
        if message_id:
            msg = await self.client.app.get_message(message_id)
            media_id = msg.media_id
            if not media_id:
                raise ValueError("Message does'nt contain any media!")
        if document:
            media = await self.client.app.upload_media(
                path=document,
                caption=message,
                thumb=thumb,
                callback=progress,
                callback_args=progress_args,
                part_size=kwargs.get("part_size"),
                task_count=kwargs.get("task_count"),
                mime_type=mime_type,
                description=(
                    file_name or os.path.basename(document) if document else None
                ),
            )
        else:
            media = Media(
                app=self.client.app,
                description=file_name,
                file_name=file_name,
                thumbnail_url=thumb,
                mime_type=mime_type,
            )
        log.debug(f"response from [upload_media]: {media}")
        response = await self.client.app.update_media_info(
            media_id=media_id, media=media
        )
        log.debug(f"response from [update_media_info]:{response}")
        if not message_id:
            return response
        response = await self.edit_message(
            message_id=message_id,
            text=message,
            inline_markup=inline_markup,
            media_id=media_id,
            **kwargs,
        )
        response.media_info = await self.client.app.get_media(media_id)
        return response

    async def edit_message(
        self,
        message_id: int,
        text: str,
        embed_message: EmbeddedMedia = None,
        inline_markup: InlineMarkup = None,
        **kwargs,
    ) -> Message:
        """Edit a message

        Parameters:
            message (``~switch.api.chat.models.Message``): The message to edit

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be edited
        """
        new_message = await self.client.app.get_message(message_id)
        if text is not None:
            new_message.message = text
        if inline_markup:
            new_message.inline_markup = inline_markup
        for key, value in kwargs.items():
            setattr(new_message, key, value)

        if embed_message is not None:
            if embed_message.thumbnail and os.path.exists(embed_message.thumbnail):
                response_media = await self.client.app.upload_media(
                    embed_message.thumbnail
                )
                embed_message.thumbnail = response_media.url
            new_message.embed_message = embed_message

        data = new_message.to_json_request()
        log.debug("Editing message %s", json.dumps(data))
        response = await self.client.put(f"{BASE_PATH}?id={message_id}", data=data)
        return self.client.build_object(Message, response.data["message"])

    async def delete_messages(self, message_ids: List[Union[int, Message]]) -> bool:
        """Delete a message

        Parameters:
            message (``int`` | ``~switch.api.chat.models.Message``): The message id or message to delete

        Returns:
            ``bool``: True if the message was deleted

        Raises:
            ``~switch.error.SwitchError``: If the message could not be deleted
        """
        message_ids = ",".join(
            [
                str(message.id if isinstance(message, Message) else message)
                for message in message_ids
            ]
        )
        log.debug(f"Deleting message {message_ids}")
        response = await self.client.delete(f"{BASE_PATH}/{message_ids}")
        log.debug(response)
        return True

    async def delete_messages_from_user(self, receiver_id: int) -> bool:
        """Delete messages from a user

        Parameters:
            recipient_id (``int``): The recipient id
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``bool``: True if the messages were deleted

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be deleted

        """
        log.debug("Deleting messages for user %s", receiver_id)
        response = await self.client.delete(
            f"{BASE_PATH}/clear-personal-messages?receiverId={receiver_id}"
        )
        return True

    async def get_messages_between_users(
        self,
        other_user_id: int,
        user_id: int = None,
        page_limit: int = 100,
        page_offset: int = 0,
    ) -> List[Message]:
        """Get messages between two users

        Parameters:
            other_user_id (``int``): The other user id.
            user_id (``int``, *optional*): The user id. Defaults to the current user id.
            page_limit (``int``, *optional*): The page limit. Defaults to 100.
            page_offset (``int``, *optional*): The page offset. Defaults to 0.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved
        """
        data = {
            "pageOffset": page_offset,
            "pageLimit": page_limit,
        }

        if user_id is None:
            user_id = self.client.user.id

        log.debug("Getting messages for user %s", other_user_id)
        response = await self.client.get(
            f"{BASE_PATH}/{user_id}/{other_user_id}?{urlencode(data)}"
        )
        return self.client.build_list(Message, response.data)

    async def forward_message(
        self,
        message_id: int | List[int],
        group_channel: Optional[Group | Channel | str] = None,
        user_id: Optional[int] = None,
    ) -> Message | List[Message]:
        """Forward a message to a group or user

        Parameters:
            message (``~switch.api.chat.models.Message`` | ``int``): The message to forward
            group_channel (``~switch.api.chat.models.Group`` | ``~switch.api.chat.models.Channel`` | ``str``, *optional*): The group or channel to forward to. Defaults to None.
            receiver_id (``str``, *optional*): The user id to forward to. Defaults to None.

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be forwarded
        """
        if isinstance(group_channel, (Group, Channel)):
            group_channel = group_channel.id
        elif group_channel is not None:
            group_channel = group_channel
        message_id_list = isinstance(message_id, list)

        if message_id_list:
            message_id = ",".join(map(str, message_id))

        q = []
        if group_channel is not None:
            q.append(f"groupChannelId={group_channel}")
        if user_id:
            q.append(f"receiverId={user_id}")

        strQuery = "&".join(q)

        log.debug("Forwarding message %s", id)
        response = await self.client.put(f"{BASE_PATH}/forward/{message_id}?{strQuery}")
        message = self.client.build_list(
            Message, [data.get("message") for data in response.data]
        )
        if message_id_list:
            return message
        return message[0] if message else None

    async def get_message(self, message_id: int) -> Message:
        """Get a message by id

        Parameters:
            message_id (``int``): The message id

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be retrieved
        """
        log.debug("Getting message %s", message_id)
        response = await self.client.get(f"{BASE_PATH}/findOne/{message_id}")
        return self.client.build_object(Message, response.data)

    async def get_group_chat_history(
        self,
        group_id: str,
        community_id: str,
        user_id: int = None,
        page_limit: int = 100,
        page_offset=0,
    ) -> GroupChatHistory:
        """Get group chat history

        Parameters:
            group_id (``str``): The group id
            community_id (``str``): The community id
            user_id (``int``, *optional*): The user id. Defaults to the current user id.
            page_limit (``int``, *optional*): The page limit. Defaults to 100.
            page_offset (``int``, *optional*): The page offset. Defaults to 0.

        Returns:
            ``~switch.api.chat.models.GroupChatHistory``: The group chat history

        Raises:
            ``~switch.error.SwitchError``: If the group chat history could not be retrieved

        """
        log.debug("Getting group chat history for group %s", group_id)
        data = {
            "communityId": community_id,
            "offset": page_offset,
            "limit": page_limit,
            "groupId": group_id,
        }
        response = await self.client.get(
            f"{BASE_PATH}/community-messages?{urlencode(data)}"
        )
        return self.client.build_object(GroupChatHistory, response.data)

    async def get_channel_chat_history(
        self,
        channel_id: str,
        community_id: str,
        user_id: int = None,
        page_limit: int = 100,
        page_offset=0,
    ) -> GroupChatHistory:
        """Get channel chat history

        Parameters:
            channel_id (``str``): The channel id
            community_id (``str``): The community id
            user_id (``int``, *optional*): The user id. Defaults to the current user id.
            page_limit (``int``, *optional*): The page limit. Defaults to 100.
            page_offset (``int``, *optional*): The page offset. Defaults to 0.

        Returns:
            ``~switch.api.chat.models.ChannelChatHistory``: The channel chat history

        Raises:
            ``~switch.error.SwitchError``: If the channel chat history could not be retrieved

        """
        log.debug("Getting channel chat history for channel %s", channel_id)
        data = {
            "communityId": community_id,
            "pageOffset": page_offset,
            "pageLimit": page_limit,
            "channelId": channel_id,
        }

        response = await self.client.get(
            f"{BASE_PATH}/community-messages?{urlencode(data)}"
        )
        return self.client.build_object(GroupChatHistory, response.data)

    async def get_community_media_files(self, community_id: str) -> List[Message]:
        """Get community media files

        Parameters:
            community_id (``str``): The community id

        Returns:
            ``List[~switch.api.chat.models.Message]``: The community media files

        Raises:
            ``~switch.error.SwitchError``: If the community media files could not be retrieved
        """
        log.debug("Getting community media files for community %s", community_id)
        response = await self.client.get(
            f"{BASE_PATH}/media?communityId={community_id}"
        )
        return self.client.build_list(Message, response.data.get("result"))

    async def get_community_media_files_by_status(
        self,
        *,
        status: Union[str, List[str]],
        community_id: str = None,
        channel_id: str = None,
        user_id: int = None,
        group_id: str = None,
    ) -> List[Message]:
        """Get community media files by status

        Parameters:
            community_id (``str``): The community id
            channel_id (``str``)
            group_id (``str``)
            user_id (``int``)
            status (``str``): The status of the media files

        Returns:
            ``List[~switch.api.chat.models.Message]``: The community media files


        Raises:
            ``~switch.error.SwitchError``: If the community media files could not be retrieved
        """
        log.debug("Getting community media files for community %s", community_id)
        if not isinstance(status, list):
            status = [status]
        params = urlencode(
            {
                x: y
                for x, y in {
                    "communityId": community_id,
                    "groupId": group_id,
                    "receiverId": user_id,
                    "status": ",".join(list(map(str, status))),
                    "channelId": channel_id,
                }.items()
                if y
            }
        )
        response = await self.client.get(f"{BASE_PATH}/mediabystatus?{params}")
        return self.client.build_list(Message, response.data.get("result"))

    async def get_user_media_files(self, user_id: int = None) -> List[Message]:
        """Get user media files


        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The user media files

        Raises:
            ``~switch.error.SwitchError``: If the user media files could not be retrieved
        """
        if user_id is None:
            user_id = self.client.user.id
        log.debug("Getting user media files for user %s", user_id)
        response = await self.client.get(f"{BASE_PATH}/media/{user_id}")
        return self.client.build_list(Message, response.data)
        # return Message.build_from_json_list(response.data)

    async def clear_conversation(self, receiver_id: int) -> bool:
        """Clear a conversation

        Parameters:
            receiver_id (``int``): The receiver id

        Returns:
            ``bool``: True if the conversation was cleared

        Raises:
            ``~switch.error.SwitchError``: If the conversation could not be cleared
        """
        log.debug("Clearing conversation %s", receiver_id)
        response = await self.client.get(
            f"{BASE_PATH}/clearconversationwith/{receiver_id}"
        )
        return True

    async def get_flag_messages(self, user_id: int = None) -> List[Message]:
        """Get flagged messages

        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The flagged messages

        Raises:
            ``~switch.error.SwitchError``: If the flagged messages could not be retrieved
        """
        if user_id is None:
            user_id = self.client.user.id

        log.debug("Get flag messages for %s", user_id)
        response = await self.client.get(f"{BASE_PATH}/flag?userId={user_id}")
        return self.client.build_list(Message, response.data)

    async def flag_message(self, message: Message | int) -> bool:
        """Flag a message

        Parameters:
            message (``~switch.api.chat.models.Message`` | ``int``): The message to flag

        Returns:
            ``bool``: True if the message was flagged

        Raises:
            ``~switch.error.SwitchError``: If the message could not be flagged
        """
        if isinstance(message, Message):
            message_id = message.id
        else:
            message_id = message
        log.debug("Flagging message %s", message_id)
        response = await self.client.post(f"{BASE_PATH}/flag?messageId={message_id}")
        return True

    async def get_unread_messages_count(self, user_id: int = None) -> int:
        """Get unread messages

        Parameters:
            user_id (``int``, *optional*): The user id. Defaults to the current user id.

        Returns:
            ``int``: The unread messages count
        """
        if user_id is None:
            user_id = self.client.user.id

        log.debug("Get unread messages count for %s", user_id)
        response = await self.client.get(
            f"{BASE_PATH}/unread-messages?userId={user_id}"
        )
        return response.data

    async def answer_inline_query(
        self, query: InlineQuery, answer: InlineQueryAnswer
    ) -> bool:
        """Answer an inline query

        Parameters:
            query (``~switch.api.chat.models.InlineQuery``): The inline query
            answer (``~switch.api.chat.models.InlineQueryAnser``): The answer

        Returns:
            ``bool``: True if the query was answered

        Raises:
            ``~switch.error.SwitchError``: If the query could not be answered
        """
        if not isinstance(query, InlineQuery):
            raise TypeError("query must be an InlineQuery instance")

        if isinstance(answer, str):
            answer = InlineQueryAnswer(
                query_id=query.query_id,
                title=answer,
                results=[],
                cache_time=0,
                is_personal=True,
                next_offset=None,
                pm_text=None,
                pm_parameter=None,
                user_id=query.user_id,
            )

        if isinstance(answer, List):
            answer = InlineQueryAnswer(
                query_id=query.query_id,
                title=None,
                results=answer,
                cache_time=0,
                is_personal=True,
                next_offset=None,
                pm_text=None,
                pm_parameter=None,
                user_id=query.user_id,
            )

        if not answer.user_id:
            answer.user_id = query.user_id

        log.debug("Answering inline query %s", query.query_id)
        response = await self.client.post(
            f"{BASE_PATH}/inline/answer", answer.to_json_request()
        )
        return response.data

    async def get_user(self, user_id: int | str = None, username: str = None) -> User:
        """Get user from user id or username"""
        if username and user_id:
            raise ValueError("'username' and 'user_id' both were provided!")
        elif user_id:
            response = await self.client.app.auth_service.get(
                f"/api/users/getUserById?userid={user_id}"
            )
        elif username:
            if username.startswith("@"):
                username = username[1:]
            response = await self.client.app.auth_service.get(
                f"/api/users/getUserByUsername?username={username}"
            )
            if not response.data:
                return
        else:
            raise ValueError("Either provide 'user_id' or 'username' to get user info.")
        return self.client.build_object(User, response.data)

    async def search_community_data(
        self,
        query: str,
        community_id: str,
        filter: Literal[
            "MESSAGE", "MEDIA", "LINK", "GROUP", "CHANNEL", "MEMBER"
        ] = "MESSAGE",
        limit: int = 10,
        page: int = 0,
    ) -> Union[List[Message], List[Group], List[Channel], List[SearchResultUser]]:
        """Search community data

        Parameters:
            query (``str``): The search query
            community_id (``str``): The community id
            filter (``str``, *optional*): The filter. Defaults to "MESSAGES".
            limit (``int``, *optional*): The limit. Defaults to 10.
            page (``int``, *optional*): The page. Defaults to 0.

        Returns:
            Union[List[Message], List[Group], List[Channel], List[SearchResultUser]]: The search results

        """
        data = {
            "searchString": query,
            "item": filter,
            "communityId": community_id,
            "limit": limit,
            "page": page,
        }
        response = await self.client.get(
            "/v1/search/community-data?{}".format(urlencode(data))
        )
        if filter in ["MESSAGE", "MEDIA", "LINK"]:
            return self.client.build_list(Message, response.data)
        elif filter == "GROUP":
            return self.client.build_list(Group, response.data)
        elif filter == "CHANNEL":
            return self.client.build_list(Channel, response.data)
        elif filter == "MEMBER":
            return self.client.build_list(SearchResultUser, response.data)
        return response.data



from typing import Optional, List
from swibots.base.switch_object import SwitchObject
from swibots.utils.types import JSONDict


class AuthUser(SwitchObject):
    def __init__(
        self,
        id: Optional[int] = None,
        user_name: Optional[str] = None,
        name: Optional[str] = None,
        creator_name: Optional[str] = None,
        verify_email: Optional[bool] = None,
        privacy: Optional[str] = None,
        is_bot: Optional[bool] = None,
        bot_privacy: Optional[str] = None,
        profile_colour: Optional[str] = None,
        otp: Optional[str] = None,
        otp_expiry: Optional[str] = None,
        bio: Optional[str] = None,
        imageurl: Optional[str] = None,
        private_imageurl: Optional[str] = None,
        gender: Optional[str] = None,
        date_of_birth: Optional[str] = None,
        medias: Optional[List[str]] = None,
        more_about_this: Optional[str] = None,
        active: Optional[bool] = None,
        parent_id: Optional[int] = None,
        created_at: Optional[str] = None,
        updated_at: Optional[str] = None,
    ):
        self.id = id
        self.user_name = user_name
        self.name = name
        self.creator_name = creator_name
        self.verify_email = verify_email
        self.privacy = privacy
        self.is_bot = is_bot
        self.bot_privacy = bot_privacy
        self.profile_colour = profile_colour
        self.otp = otp
        self.otp_expiry = otp_expiry
        self.bio = bio
        self.imageurl = imageurl
        self.private_imageurl = private_imageurl
        self.gender = gender
        self.date_of_birth = date_of_birth
        self.medias = medias
        self.more_about_this = more_about_this
        self.active = active
        self.parent_id = parent_id
        self.created_at = created_at
        self.updated_at = updated_at

    def from_json(self, data: Optional[JSONDict]) -> "AuthUser":
        super().from_json(data)
        if data is not None:
            self.id = data.get("id")
            self.user_name = data.get("user_name")
            self.name = data.get("name")
            self.creator_name = data.get("creator_name")
            self.verify_email = data.get("verifyEmail")
            self.privacy = data.get("privacy")
            self.is_bot = data.get("is_bot")
            self.bot_privacy = data.get("bot_privacy")
            self.profile_colour = data.get("profile_colour")
            self.otp = data.get("otp")
            self.otp_expiry = data.get("otp_expiry")
            self.bio = data.get("bio")
            self.imageurl = data.get("imageurl")
            self.private_imageurl = data.get("private_imageurl")
            self.gender = data.get("gender")
            self.date_of_birth = data.get("date_of_birth")
            self.medias = []
            for _ in range(1, 6):
                if media := data.get(f"media{_}"):
                    self.medias.append(media)
            self.more_about_this = data.get("more_about_this")
            self.active = data.get("active")
            self.parent_id = data.get("parent_id")
            self.created_at = data.get("createdAt")
            self.updated_at = data.get("updatedAt")

        return self

    def to_json(self) -> JSONDict:
        return {
            "id": self.id,
            "user_name": self.user_name,
            "name": self.name,
            "creator_name": self.creator_name,
            "verifyEmail": self.verify_email,
            "privacy": self.privacy,
            "is_bot": self.is_bot,
            "bot_privacy": self.bot_privacy,
            "profile_colour": self.profile_colour,
            "otp": self.otp,
            "otp_expiry": self.otp_expiry,
            "bio": self.bio,
            "imageurl": self.imageurl,
            "private_imageurl": self.private_imageurl,
            "gender": self.gender,
            "date_of_birth": self.date_of_birth,
            "media1": self.media1,
            "media2": self.media2,
            "media3": self.media3,
            "media4": self.media4,
            "media5": self.media5,
            "more_about_this": self.more_about_this,
            "active": self.active,
            "parent_id": self.parent_id,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
        }



import os
from typing import Optional, List
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject
from .embed_inline_field import EmbedInlineField


class EmbeddedMedia(SwitchObject):
    def __init__(
        self,
        thumbnail: str,
        description: Optional[str] = None,
        footer_icon: Optional[str] = None,
        footer_title: Optional[str] = None,
        header_icon: Optional[str] = None,
        header_name: Optional[str] = None,
        inline_fields: Optional[List[List["EmbedInlineField"]]] = None,
        title: Optional[str] = None,
    ):
        super().__init__()
        self.thumbnail = thumbnail
        self.description = description
        self.footer_icon = footer_icon
        self.footer_title = footer_title
        self.header_icon = header_icon
        self.header_name = header_name
        self.inline_fields = inline_fields
        self.title = title

    def data_to_request(self):
        form_data = {
            "isEmbedMessage": "true",
            "embedMessage.title": self.title,
            "embedMessage.description": self.description,
            "embedMessage.headerName": self.header_name,
            "embedMessage.headerIcon": self.header_icon,
            "embedMessage.footerTitle": self.footer_title,
            "embedMessage.footerIcon": self.footer_icon,
        }
        for i, kb in enumerate(self.inline_fields or []):
            for j, b in enumerate(kb):
                key = "embedMessage.inlineFields[{0}][{1}].key".format(i, j)
                form_data[key] = b.key
                title_key = "embedMessage.inlineFields[{0}][{1}].title".format(i, j)
                form_data[title_key] = b.title
                icon_key = "embedMessage.inlineFields[{0}][{1}].icon".format(i, j)
                form_data[icon_key] = b.icon
        return form_data

    def from_json(self, data: JSONDict = None) -> "EmbeddedMedia":
        if data is not None:
            self.thumbnail = data.get("coverPic")
            self.description = data.get("description")
            self.footer_icon = data.get("footerIcon")
            self.footer_title = data.get("footerTitle")
            self.header_icon = data.get("headerIcon")
            self.header_name = data.get("headerName")
            self.title = data.get("title")
            self.inline_fields = [
                [
                    EmbedInlineField(
                        key=x.get("key"), icon=x.get("icon"), title=x.get("title")
                    )
                    for x in row
                ]
                for row in data.get("inlineFields", [])
            ]
        return self

    def to_json(self) -> JSONDict:
        return {
            "coverPic": self.thumbnail if isinstance(self.thumbnail, str) else None,
            "description": self.description,
            "footerIcon": self.footer_icon,
            "footerTitle": self.footer_title,
            "headerIcon": self.header_icon,
            "headerName": self.header_name,
            "inlineFields": [
                [x.to_json() for x in row if x] for row in (self.inline_fields or [])
            ],
            "title": self.title,
        }



from typing import Type, TypeVar, Union
import swibots, asyncio
from swibots.api.community import Channel, Group
from swibots.api.chat import Message
from swibots.api.common.models import User
from swibots.api.chat.events import MessageEvent


class ListenMessages:
    async def listen_messages(
        self: "swibots.ApiClient", chat_id: str, count: int = 1, timeout: int = 60
    ):
        messageBox = []

        async def _on_message(ctx: swibots.BotContext[MessageEvent]):
            msg = ctx.event.message
            if msg.chat_id == chat_id:
                messageBox.append(msg)

        async def getMessages():
            while not len(messageBox) == count:
                await asyncio.sleep(0)

        app = self.bots_service.app

        from swibots.bots.handlers.message_handler import MessageHandler

        handler = MessageHandler(_on_message)
        try:
            app.add_handler(handler)
            await asyncio.wait_for(getMessages(), timeout=timeout)
        except TimeoutError as er:
            raise er
        finally:
            app.remove_handler(handler)

        if count == 1:
            return messageBox[0]
        return messageBox



from typing import Type, TypeVar
import swibots
from swibots.api.chat.models import Message


class DeleteMessagesFromUser:
    async def delete_messages_from_user(
        self: "swibots.ApiClient", user_id: int
    ) -> bool:
        """Delete all messages from a user

        Parameters:
            user_id (``int``): The user id

        Returns:
            ``bool``: Whether the messages were deleted

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be deleted

        This method does the same as :meth:`~switch.api.chat.controllers.MessageController.delete_messages_from_user`.
        """
        return await self.chat_service.messages.delete_messages_from_user(user_id)



import swibots
from enum import Enum
from swibots.base import SwitchObject
from swibots.utils.types import JSONDict
from typing import Dict, Any


class ScreenType(Enum):
    BOTTOM = "bottom"
    SCREEN = "screen"


class TextSize(Enum):
    SMALL = "x"
    MEDIUM = "x_large"
    LARGE = "xx_large"
    BODY = "body"
    BOLD = "bold_body"
    MARKDOWN = "markdown"


class Expansion(Enum):
    DEFAULT = "default_expansion"
    # TODO: Remove FLEXIBLE, EXPAND
    FLEXIBLE = "flexible_expansion"
    EXPAND = "expanded_expansion"

    VERTICAL = "expanded_expansion"
    HORIZONTAL = "flexible_expansion"


class Component(SwitchObject):
    type = None


class Icon(Component):
    type = "icon"

    def __init__(
        self, url: str, dark_url: str = None, text: str = "", callback_data: str = None
    ):
        self.url = url
        self.dark_url = dark_url or url
        self.text = text
        self.callback_data = callback_data

    def to_json(self) -> Dict[str, Any]:
        return {
            "type": self.type,
            "name": self.url,
            "darkIcon": self.dark_url,
            "alt": self.text,
            "callbackData": self.callback_data,
        }


class Text(Component):
    type = "text"

    def __init__(
        self,
        text: str,
        size: TextSize = TextSize.BODY,
        opacity: float = 1,
        color: str = None,
        max_size: bool = None,
    ):
        self.text = text
        self.size = size
        self.opacity = opacity
        self.color = color
        self.max_size = max_size

    def to_json(self) -> Dict[str, Any]:
        return {
            "type": self.type,
            "text": self.text,
            "textType": self.size.value,
            "color": self.color,
            "mainAxisSize": "max" if self.max_size else "min",
        }


class Image(Component):
    type = "image"

    def __init__(
        self,
        url: str,
        callback_data: str = None,
        dark_url: str = None,
        max_size: bool = None,
    ):
        self.url = url
        self.callback_data = callback_data
        self.dark_url = dark_url or url
        self.max_size = max_size

    def to_json(self):
        data = {
            "type": self.type,
            "mediaUrl": self.url,
            "darkMediaUrl": self.dark_url,
            "mainAxisSize": "max" if self.max_size else "min",
        }
        if self.callback_data:
            data["callbackData"] = self.callback_data
        return data


class Spacer(Component):
    type = "spacer"

    def __init__(self, x: int = 0, y: int = 0):
        self.x = x
        self.y = y

    def to_json(self):
        return {"type": self.type, "x": self.x, "y": self.y}


class Badge(Component):
    type = "badge"

    def __init__(self, text: str, background: str = None, text_color: str = None):
        self.text = text
        self.background = background
        self.text_color = text_color

    def to_json(self):
        return {
            "type": self.type,
            "text": self.text,
            "textColor": self.text_color,
            "background": self.background,
        }



import swibots

from swibots.api.community.models import Role
from typing import List


class RoleMethods:
    """
    Methods for managing roles in a community.

    Args:
        client (swibots.ApiClient): The API client.
    """

    async def add_role(
        self: "swibots.ApiClient",
        community_id: str,
        name: str,
        colour: str,
    ) -> Role:
        """
        Add a new role to a community.

        Args:
            community_id (str): The ID of the community.
            name (str): The name of the role.
            colour (str): The color of the role.

        Returns:
            :obj:`swibots.api.community.models.Role`: The new role.
        """

        return await self.community_service.roles.add_role(
            community_id=community_id, role_colour=colour, role_name=name
        )

    async def get_roles(self: "swibots.ApiClient", community_id: str) -> List[Role]:
        """
        Retrieve all roles in a community.

        Args:
            community_id (str): The ID of the community.

        Returns:
            List[:obj:`swibots.api.community.models.Role`]: A list of all roles in the community.
        """

        return await self.community_service.roles.get_roles(community_id)

    async def delete_role(self: "swibots.ApiClient", role_id: str):
        """
        Delete a role from a community.

        Args:
            role_id (str): The ID of the role to delete.
        """

        return await self.community_service.roles.delete_role(role_id)

    async def update_role(
        self: "swibots.ApiClient",
        community_id: str,
        role_id: int,
        role_colour: str,
        role_name: str,
    ):
        """
        Update a role in a community.

        Args:
            community_id (str): The ID of the community.
            role_id (int): The ID of the role to update.
            role_colour (str): The new color of the role.
            role_name (str): The new name of the role.

        Returns:
            :obj:`swibots.api.community.models.Role`: The updated role.
        """

        return await self.community_service.roles.update_role(
            community_id, role_id, role_colour, role_name
        )



from typing import TYPE_CHECKING, BinaryIO, Callable, List, Optional, Union
from re import Match
import swibots
from io import BytesIO
from swibots.base import SwitchObject
from swibots.api.common import User, Media, EmbeddedMedia
from swibots.api.community import Community, Channel, Group
from swibots.utils.types import JSONDict
from swibots.types import MediaType
from .inline_markup import InlineMarkup


class Message(
    SwitchObject,
):
    def __init__(
        self,
        app: "swibots.App" = None,
        id: int = None,
        user_id: int = None,
        user: "User" = None,
        receiver_id: int = None,
        receiver: "User" = None,
        message: str = None,
        matches: List[Match] = None,
        sent_date: int = None,
        status: int = None,
        request_id: int = None,
        button_name: str = None,
        button_pressed_id: int = None,
        callback_data: str = None,
        channel_chat: bool = None,
        channel_id: int = None,
        channel: "Channel" = None,
        command_name: str = None,
        community_id: int = None,
        community: "Community" = None,
        user_session_id: Optional[str] = None,
        embed_message: "EmbeddedMedia" = None,
        is_embed_message: str = None,
        edit: bool = None,
        flag: int = None,
        forward: bool = None,
        group_chat: bool = None,
        group_id: int = None,
        group: "Group" = None,
        information: str = None,
        inline_markup: "InlineMarkup" = None,
        is_read: bool = None,
        is_document: bool = None,
        media_link: str = None,
        mentioned_ids: List[int] = None,
        personal_chat: bool = None,
        pinned: bool = None,
        reactions: List[str] = None,
        replied_message: "Message" = None,
        replied_to_id: int = None,
        replied_to: "Message" = None,
        replies: List["Message"] = None,
        reply_count: int = None,
        media_id: int = None,
        link: str = None,
        media_info: Media = None,
        cached_media: Media = None,
        scheduled_at: Optional[int] = None,
        sticker_pack_id: Optional[str] = None,
        command_details: Optional[dict] = None,
        **kwargs,
    ):
        super().__init__(app=app)
        self.id = id
        self.user_id = user_id
        self.user = user
        self.receiver_id = receiver_id
        self.receiver = receiver
        self.message = message
        self.matches = matches
        self.sent_date = sent_date
        self.status = status
        self.request_id = request_id
        self.button_name = button_name
        self.button_pressed_id = button_pressed_id
        self.callback_data = callback_data
        self.channel_chat = channel_chat
        self.channel_id = channel_id
        self.channel = channel
        self.command_name = command_name
        self.community_id = community_id
        self.community = community
        self.edit = edit
        self.flag = flag
        self.forward = forward
        self.group_chat = group_chat
        self.group_id = group_id
        self.group = group
        self.information = information
        self.inline_markup = inline_markup
        self.is_read = is_read
        self.is_document = is_document
        self.media_link = media_link
        self.media_id = media_id
        self.media_info = media_info
        self.mentioned_ids = mentioned_ids
        self.personal_chat = personal_chat
        self.pinned = pinned
        self.reactions = reactions
        self.replied_message = replied_message
        self.replied_to_id = replied_to_id
        self.replied_to = replied_to
        self.replies = replies
        self.reply_count = reply_count
        self.cached_media = cached_media
        self.link = link
        self.embed_message = embed_message
        self.is_embed_message = is_embed_message
        self.scheduled_at = scheduled_at
        self.user_session_id = user_session_id
        self.sticker_pack_id = sticker_pack_id
        self.command_details = command_details
        self.__dict__.update(**kwargs)

    def to_json_request(self) -> JSONDict:
        return {
            "id": self.id,
            "message": self.message,
            "receiverId": self.receiver_id,
            "requestId": self.request_id,
            "userId": self.user_id,
            "communityId": self.community_id,
            "groupId": self.group_id,
            "channelId": self.channel_id,
            "inline_markup": (
                self.inline_markup.to_json_request() if self.inline_markup else None
            ),
            "callback_data": self.callback_data,
            "repliedTo": self.replied_to_id,
            "mediaLink": self.media_link,
            "status": 4 if self.embed_message else self.status,
            "embedMessage": (
                self.embed_message.to_json() if self.embed_message else None
            ),
            "isEmbedMessage": self.is_embed_message or bool(self.embed_message),
            "cachedMedia": self.cached_media.to_json() if self.cached_media else None,
            "mediaId": self.media_id,
            "mediaInfo": self.media_info.to_json() if self.media_info else None,
            "userSessionId": self.user_session_id,
            "link": self.link,
            "commandDetails": self.command_details,
        }

    def to_form_data(self):
        form_data = {}
        if self.user_session_id is not None:
            form_data["userSessionId"] = self.user_session_id
        if self.user_id is not None:
            form_data["userId"] = self.user_id
        if self.receiver_id is not None:
            form_data["receiverId"] = self.receiver_id
        if self.request_id is not None:
            form_data["requestId"] = self.request_id
        if self.community_id is not None:
            form_data["communityId"] = self.community_id
        if self.channel_id is not None:
            form_data["channelId"] = self.channel_id
        if self.group_id is not None:
            form_data["groupId"] = self.group_id
        if self.replied_to_id is not None:
            form_data["repliedTo"] = self.replied_to_id
        if self.message is not None:
            form_data["message"] = self.message
        if self.media_link is not None:
            form_data["mediaLink"] = self.media_link
        if self.status is not None:
            form_data["status"] = self.status
        if self.is_document is not None:
            form_data["isDocument"] = self.is_document
        if self.inline_markup is not None:
            form_data.update(self.inline_markup.to_form_data())
        if self.scheduled_at is not None:
            form_data["scheduledAt"] = self.scheduled_at
        if self.media_info is not None:
            form_data.update(self.media_info.to_form_data())
            self.media_link = self.media_info.url
        return form_data

    def to_json(self) -> JSONDict:
        replied = self.replied_message or self.replied_to
        return {
            "buttonName": self.button_name,
            "buttonPressedId": self.button_pressed_id,
            "callback_data": self.callback_data,
            "cachedMedia": self.cached_media.to_json() if self.cached_media else None,
            "channelChat": self.channel_chat,
            "channelId": self.channel_id,
            "commandName": self.command_name,
            "communityId": self.community_id,
            "edit": self.edit,
            "flag": self.flag,
            "forward": self.forward,
            "groupChat": self.group_chat,
            "groupId": self.group_id,
            "id": self.id,
            "information": self.information,
            "embedMessage": (
                self.embed_message.to_json() if self.embed_message else None
            ),
            "isEmbedMessage": self.is_embed_message,
            "inline_markup": (
                self.inline_markup.to_json() if self.inline_markup else None
            ),
            "isRead": self.is_read,
            "isDocument": self.is_document,
            "mediaLink": self.media_link,
            "mentionedIds": self.mentioned_ids,
            "message": self.message,
            "personalChat": self.personal_chat,
            "pinned": self.pinned,
            "reactions": self.reactions,
            "receiverId": self.receiver_id,
            "mediaId": self.media_id,
            "mediaInfo": self.media_info.to_json() if self.media_info else None,
            "repliedMessage": replied.to_json() if replied else None,
            "repliedTo": self.replied_to_id,
            "replies": self.replies,
            "replyCount": self.reply_count,
            "requestId": self.request_id,
            "sentDate": self.sent_date,
            "status": self.status,
            "userId": self.user_id,
            "userSessionId": self.user_session_id,
            "link": self.link,
            "commandDetails": self.command_details,
        }

    def from_json(self, data: Optional[JSONDict]) -> "Message":
        if data is not None:
            self.button_name = data.get("buttonName")
            self.button_pressed_id = data.get("buttonPressedId")
            self.callback_data = data.get("callback_data")
            self.channel_chat = data.get("channelChat")
            self.channel_id = data.get("channelId")
            self.channel = Channel.build_from_json(data.get("channel"), self.app)
            self.command_name = data.get("commandName")
            self.community_id = data.get("communityId")
            self.community = Community.build_from_json(data.get("community"), self.app)
            self.edit = data.get("edit")
            self.flag = data.get("flag")
            self.forward = data.get("forward")
            self.group_chat = data.get("groupChat")
            self.group_id = data.get("groupId")
            self.group = Group.build_from_json(data.get("group"), self.app)
            self.id = data.get("id")
            self.information = data.get("information")
            self.inline_markup = InlineMarkup.build_from_json(
                data.get("inline_markup"), self.app
            )
            self.link = data.get("link")
            self.is_read = data.get("isRead")
            self.is_document = data.get("isDocument")
            self.media_link = data.get("mediaLink")
            self.media_id = data.get("mediaId")
            self.media_info: Media = Media.build_from_json(
                data.get("mediaInfo"), self.app
            )
            if self.media_info:
                if not self.media_id:
                    self.media_id = self.media_info.id
                if not self.media_link:
                    self.media_link = self.media_info.url
            self.cached_media = Media.build_from_json(data.get("cachedMedia"), self.app)
            self.mentioned_ids = data.get("mentionedIds")
            self.message = data.get("message")
            self.personal_chat = data.get("personalChat")
            self.pinned = data.get("pinned")
            self.reactions = data.get("reactions")
            self.command_details = data.get("commandDetails")
            receiver_id = data.get("receiverId", 0)
            if receiver_id == "null":
                receiver_id = 0
            self.receiver_id = int(receiver_id)
            self.replied_to = Message.build_from_json(
                data.get("repliedMessage"), self.app
            )
            self.replied_to_id = data.get("repliedTo")
            self.replies = data.get("replies")
            self.reply_count = data.get("replyCount")
            self.request_id = data.get("requestId")
            self.sent_date = data.get("sentDate")
            self.status = data.get("status")

            self.user = User.build_from_json(data.get("senderInfo"), self.app)
            self.user_id = int(data.get("userId", 0))
            self.embed_message = EmbeddedMedia.build_from_json(
                data.get("embedMessage"), self.app
            )
            self.is_embed_message = data.get("isEmbedMessage") == "true" or bool(
                self.embed_message
            )
            self.user_session_id = data.get("userSessionId")
            self.sticker_pack_id = data.get("stickerPackId")
        return self

    @property
    def chat_id(self):
        return self.user_session_id or self.channel_id or self.group_id or self.user_id

    @property
    def outgoing(self) -> bool:
        return self.user_id == self.app.user.id

    @property
    def is_media(self):
        # 1 = IMAGE
        # 2 = Video
        # 3 = AUDIO
        # 7 = FILE
        return self.status in [1, 2, 3, 7]

    @property
    def video(self):
        """Whether message includes video"""
        return self.media_info and self.media_info.media_type == MediaType.VIDEO.value

    @property
    def photo(self):
        """Whether message includes photo"""
        return self.media_info and self.media_info.media_type == MediaType.IMAGE.value

    @property
    def audio(self):
        """Whether message includes audio as media"""
        return self.media_info and self.media_info.media_type == MediaType.AUDIO.value

    @property
    def document(self):
        """Whether message includes document"""
        return (
            self.media_info and self.media_info.media_type == MediaType.DOCUMENT.value
        )

    @property
    def sticker(self):
        """Whether message is sticker"""
        return bool(self.sticker_pack_id)

    async def get_replies(self) -> List["Message"]:
        if self.reply_count <= 0:
            return []
        if self.replies is None:
            self.replies = await self.app.get_message_replies(self.id)
        return self.replies

    async def get_replied_message(self) -> "Message":
        if self.replied_message:
            return self.replied_message
        if self.replied_to_id is None or self.replied_to_id <= 0:
            return None
        if self.replied_to is None:
            self.replied_to = await self.app.get_message(self.replied_to_id)
        return self.replied_to

    def _get_receiver_id(self):
        """Get receiver id to send message"""
        if not self.personal_chat:
            return
        return self.user_id if self.user_id != self.app.user.id else self.receiver_id

    ### API Methods ###

    async def respond(
        self,
        message: str,
        embed_message: EmbeddedMedia = None,
        inline_markup: InlineMarkup = None,
        **kwargs,
    ) -> "Message":
        return await self.app.send_message(
            message=message,
            community_id=self.community_id,
            group_id=self.group_id,
            channel_id=self.channel_id,
            user_id=self._get_receiver_id(),
            user_session_id=self.user_session_id,
            embed_message=embed_message,
            inline_markup=inline_markup,
            **kwargs,
        )

    send = respond

    async def delete(self) -> None:
        return await self.app.delete_messages([self.id])

    async def reply_text(
        self,
        message: str,
        embed_message: EmbeddedMedia = None,
        inline_markup: InlineMarkup = None,
        quote: bool = None,
        **kwargs,
    ) -> "Message":
        if quote == None:
            quote = True if self.personal_chat else False

        return await self.app.send_message(
            message,
            community_id=self.community_id,
            group_id=self.group_id,
            channel_id=self.channel_id,
            user_id=self._get_receiver_id(),
            reply_to_message_id=self.id if quote else None,
            user_session_id=self.user_session_id,
            embed_message=embed_message,
            inline_markup=inline_markup,
            **kwargs,
        )

    async def reply_media(
        self,
        document: Union[str, BytesIO],
        message: str = "",
        thumb: Union[str, BytesIO] = None,
        progress=None,
        progress_args=None,
        inline_markup: InlineMarkup = None,
        quote: bool = None,
        **kwargs,
    ) -> "Message":
        """reply media to the message"""
        if quote == None:
            quote = True if self.personal_chat else False
        return await self.app.send_media(
            message=message,
            document=document,
            community_id=self.community_id,
            group_id=self.group_id,
            channel_id=self.channel_id,
            user_id=self._get_receiver_id(),
            thumb=thumb,
            user_session_id=self.user_session_id,
            reply_to_message_id=self.id if quote else None,
            progress=progress,
            progress_args=progress_args,
            inline_markup=inline_markup,
            **kwargs,
        )

    async def edit_inline_markup(self, inline_markup: InlineMarkup = None):
        """Edit inline markup of message"""
        if not inline_markup:
            inline_markup = InlineMarkup()
        return await self.edit_text(text=None, inline_markup=inline_markup)

    async def edit_media(
        self,
        document: str,
        message: Optional[str] = None,
        inline_markup: Optional[InlineMarkup] = None,
        **kwargs,
    ):
        """Bound method of client.edit_media

        Args:
           document: path to file
           message: message caption
           inline_markup: InlineMarkup
        """
        return await self.app.edit_media(
            message_id=self.id,
            media_id=self.media_id,
            document=document,
            message=message,
            inline_markup=inline_markup,
            **kwargs,
        )

    async def reply_document(
        self,
        document: Union[str, BytesIO],
        message: str = "",
        thumb: Union[str, BytesIO] = None,
        progress=None,
        progress_args=None,
        inline_markup: InlineMarkup = None,
        quote: bool = None,
        **kwargs,
    ) -> "Message":
        """Reply document to the message"""
        return await self.reply_media(
            message=message,
            document=document,
            media_type=MediaType.DOCUMENT.value,
            thumb=thumb,
            progress=progress,
            progress_args=progress_args,
            inline_markup=inline_markup,
            quote=quote,
            **kwargs,
        )

    async def reply_audio(
        self,
        audio: Union[str, BytesIO],
        caption: str = "",
        inline_markup: InlineMarkup = None,
        quote: bool = None,
        **kwargs,
    ):
        """Reply audio to message!"""
        return await self.reply_media(
            message=caption,
            document=audio,
            media_type=MediaType.AUDIO.value,
            inline_markup=inline_markup,
            quote=quote,
            **kwargs,
        )

    async def edit_text(
        self,
        text: str,
        embed_message: EmbeddedMedia = None,
        inline_markup: Optional[InlineMarkup] = None,
        **kwargs,
    ) -> "Message":
        if not inline_markup:
            inline_markup = self.inline_markup

        return await self.app.edit_message_text(
            self.id,
            text,
            embed_message=embed_message,
            inline_markup=inline_markup,
            **kwargs,
        )

    async def listen(self, *args, **kwargs):
        """Listen for messages in current chat"""
        return await self.app.listen_messages(self.chat_id, *args, **kwargs)

    async def forward_to(
        self,
        group_channel: Union[Group, Channel, str] = None,
        user_id: Optional[int] = None,
    ):
        return await self.app.forward_message(self.id, group_channel, user_id)

    async def download(
        self,
        file_name: str = None,
        in_memory: bool = False,
        block: bool = True,
        progress: Callable = None,
        progress_args: tuple = (),
    ) -> Optional[Union[BinaryIO, bytes]]:
        if not file_name and self.media_info:
            file_name = self.media_info.description
        return await self.app.download_media(
            self, file_name, in_memory, block, progress, progress_args
        )



from swibots.utils.types import JSONDict
from .types import Image, Component
from typing import List, Any


class Carousel(Component):
    type = "carousel"

    def __init__(
        self,
        images: List[Image],
        title: str = "",
        subtitle: str = "",
        max_size: bool = None,
    ):
        self.title = title
        self.subtitle = subtitle
        self.images = images
        self.max_size = max_size

    def to_json(self):
        data = {
            "type": self.type,
            "components": [image.to_json() for image in self.images],
            "title": self.title,
            "subTitle": self.subtitle,
            "mainAxisSize": "max" if self.max_size else "min",
        }
        return data



import logging
from typing import TYPE_CHECKING, Optional, List

from swibots.api.common.models import User
from swibots.api.community.models import RoleMember


if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community/member"


class RoleMemberController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def get_members(self, role_id: str) -> List[RoleMember]:
        """Get all members by role id"""
        response = await self.client.get(f"{BASE_PATH}/getAll?roleId={role_id}")
        return self.client.build_list(RoleMember, response.data.get("result"))

    async def add_member_to_role(
        self,
        community_id: str,
        member_id: int,
        role_ids: List[int],
    ) -> bool:
        data = {"communityId": community_id, "roleIds": role_ids, "memberId": member_id}
        response = await self.client.post(f"{BASE_PATH}/add", data=data)
        return response.data.get("success", False)

    async def delete_role_member(
        self,
        id: Optional[int] = None,
        member_id: Optional[int] = None,
        role_id: Optional[int] = None,
    ) -> bool:
        """delete member from role by id"""
        if id:
            response = await self.client.delete(f"{BASE_PATH}/delete/{id}")
        elif member_id and role_id:
            response = await self.client.delete(
                f"{BASE_PATH}/delete?memberId={member_id}&roleId={role_id}"
            )
        else:
            raise ValueError(
                "Either provide 'id' or 'member_id with role_id' to  delete member from the role."
            )
        return response.data.get("success", False)



from typing import TYPE_CHECKING, Optional
from swibots.api.api_client import ApiClient
from swibots.api.auth.models.auth_user import AuthUser
from swibots.api.bot.models import BotInfo, BotCommand
from swibots.api.chat.models import Message

if TYPE_CHECKING:
    from swibots import Client


class Bot(AuthUser, ApiClient):
    def __init__(self, app: Optional["Client"] = None):
        super().__init__(app)
        self._app = app
        self._info: BotInfo = None

        # copy methods
        self.add_handler = app.add_handler
        self.remove_handler = app.remove_handler
        self.handlers = app.handlers
        self.update_bot_commands = self.app.update_bot_commands
        self.set_bot_commands = self.app.set_bot_commands
        self.delete_bot_commands = self.app.delete_bot_commands

    @property
    def app(self) -> "Client":
        return self._app

    @app.setter
    def app(self, value: "Client"):
        self._app = value
        # copy the api client
        self._chat_client = value.chat_service
        self._auth_client = value.auth_service
        self._community_client = value.community_service
        self._bot_client = value.bots_service

    @property
    def info(self) -> BotInfo:
        """Get the bot info"""
        return self._info

    @info.setter
    def info(self, value: BotInfo):
        """Set the bot info"""
        self._info = value

    @property
    def commands(self) -> list[BotCommand]:
        """Get the bot commands"""
        return self._info.commands or []



import swibots

from swibots.api.community.models import Channel


class UnbanUser:
    async def unban_user(
        self: "swibots.ApiClient",  community_id: str, user_id: str
    ):
        """Unbans a user in a community.

        Args:
            community_id (str): The ID of the community.
            user_id (str): The ID of the user to unban.

        Returns:
            bool: Whether the unban was successful.
        """
        return await self.community_service.ban.unban_user(community_id, user_id)



import swibots
from typing import Optional, Union
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject
from swibots.types import GetMediaType


class Media(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        id: Optional[int] = None,
        caption: Optional[str] = None,
        checksum: Optional[str] = None,
        description: Optional[str] = None,
        thumbnail_url: Optional[str] = None,
        type_name: Optional[str] = None,
        source_id: Optional[bool] = None,
        media_type: Optional[Union[str, int]] = None,
        mime_type: Optional[str] = 0,
        file_name: Optional[bool] = None,
        file_size: Optional[bool] = None,
        url: Optional[bool] = None,
        owner_id: Optional[int] = None,
        owner_type: Optional[str] = None,
        premium: Optional[bool] = False,
    ):
        super().__init__(app)
        self.id = id
        self.caption = caption
        self.description = description
        self.checksum = checksum
        self.thumbnail_url = thumbnail_url
        self.source_id = source_id
        self.type_name = type_name
        self.media_type = media_type
        self.mime_type = mime_type
        self.file_name = file_name
        self.file_size = file_size
        self.owner_id = owner_id
        self.url = url
        self.owner_type = owner_type
        self.premium = premium

    @property
    def is_sticker(self) -> bool:
        return 200 <= (self.media_type) <= 202

    def to_form_data(self):
        return {
            "mediaInfo.caption": self.caption,
            "mediaInfo.description": self.description,
            "mediaInfo.checksum": self.checksum,
            "mediaInfo.fileName": self.file_name,
            "mediaInfo.fileSize": self.file_size,
            "mediaInfo.mimeType": self.mime_type,
            "mediaInfo.mediaType": self.media_type,
            "mediaInfo.url": self.url,
            "mediaInfo.thumbnailUrl": self.thumbnail_url,
            "mediaInfo.sourceUri": self.source_id,
            "mediaInfo.typeName": self.type_name,
            "mediaInfo.premium": self.premium,
        }

    def to_json(self) -> JSONDict:
        return {
            x: y
            for x, y in {
                "id": self.id,
                "caption": self.caption,
                "checksum": self.checksum,
                "description": self.description,
                "thumbnailUrl": self.thumbnail_url,
                "sourceUri": self.source_id,
                "mediaType": self.media_type,
                "mimeType": self.mime_type,
                "fileName": self.file_name,
                "fileSize": self.file_size,
                "typeName": self.type_name,
                "ownerId": self.owner_id,
                "ownerType": self.owner_type,
                "downloadUrl": self.url,
                "url": self.url,
                "premium": self.premium,
            }.items()
            if y
        }

    def from_json(self, data: Optional[JSONDict] = None) -> "Media":
        if data is not None:
            self.id = data.get("id")
            self.checksum = data.get("checksum")
            self.caption = data.get("caption")
            self.description = data.get("description")
            self.thumbnail_url = data.get("thumbnailUrl")
            self.source_id = data.get("sourceUri")
            self.media_type = data.get("mediaType")
            self.mime_type = data.get("mimeType")
            self.file_name = data.get("fileName")
            self.file_size = data.get("fileSize")
            self.url = data.get("downloadUrl")
            self.owner_id = data.get("ownerId")
            self.owner_type = data.get("ownerType")
            if isinstance(self.owner_id, str) and self.owner_id.isdigit():
                self.owner_id = int(self.owner_id)
            self.premium = data.get("premium", False)
        return self

    def to_update_request(self):
        return {
            "id": self.id,
            "checksum": self.checksum,
            "description": self.description,
            "thumbnailUrl": self.thumbnail_url,
            "sourceUri": self.source_id,
            "mediaType": GetMediaType(self.media_type),
            "mimeType": self.mime_type,
            "fileName": self.file_name,
            "fileSize": self.file_size,
            "url": self.url,
            "premium": self.premium,
        }

    async def edit(
        self,
        thumb_url: Optional[str] = None,
        url: Optional[str] = None,
        media_type: Optional[str] = None,
        file_name: Optional[str] = None,
        media: "Media" = None,
    ):
        """Update media Info

        Args:
          file_name: Caption of media
          media: Media: new media object
        """
        return await self.app.update_media_info(
            self.id,
            media=media,
            thumb_url=thumb_url,
            url=url,
            file_name=file_name,
            media_type=media_type,
        )



from typing import List, Optional
import swibots
from swibots.base import SwitchObject
from swibots.api.common import User
from swibots.api.community import Community, Channel, Group
from .inline_query_answer import InlineQueryAnswer

from swibots.utils.types import JSONDict


class InlineQuery(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        user_id: int = None,
        user: "User" = None,
        # receiver_id: int = None,
        # receiver: "User" = None,
        community_id: int = None,
        community: "Community" = None,
        group_id: int = None,
        group: "Group" = None,
        channel_id: int = None,
        channel: "Channel" = None,
        query_id: Optional[str] = None,
        offset: Optional[str] = None,
        query: Optional[str] = None,
    ):
        super().__init__(app)
        self.user_id = user_id
        self.user = user
        # self.receiver_id = receiver_id
        # self.receiver = receiver
        self.community_id = community_id
        self.community = community
        self.group_id = group_id
        self.group = group
        self.channel_id = channel_id
        self.channel = channel
        self.query_id = query_id
        self.offset = offset
        self.query = query

    def to_json(self) -> JSONDict:
        return {
            "text": self.text,
            "url": self.url,
            "callbackData": self.callback_data,
        }

    def from_json(self, data: JSONDict) -> "InlineQuery":
        if data is not None:
            self.user_id = data.get("userId")
            self.user = User.build_from_json(data.get("user"), self.app)
            # self.receiver_id = data.get("receiverId")
            # self.receiver = User.build_from_json(
            #     data.get("receiver"), self.app)
            self.community_id = data.get("communityId")
            self.community = Community.build_from_json(data.get("community"), self.app)
            self.group_id = data.get("groupId")
            self.group = Group.build_from_json(data.get("group"), self.app)
            self.channel_id = data.get("channelId")
            self.channel = Channel.build_from_json(data.get("channel"), self.app)
            self.query_id = data.get("id")
            self.offset = data.get("offset")
            self.query = data.get("query")
        return self

    async def answer(self, response: "InlineQueryAnswer"):
        return await self.app.answer_inline_query(self, response)



from typing import Callable, Optional
import swibots
from swibots.bots.filters.filter import Filter


class OnGroupCreated:
    def on_group_created(
        self: "swibots.Client" = None, filter: Optional[Filter] = None
    ) -> Callable:
        """Decorator for handling group creations."""

        def decorator(func: Callable) -> Callable:
            if isinstance(self, swibots.Client):
                self.add_handler(
                    swibots.bots.handlers.GroupCreatedHandler(func, filter)
                )

            return func

        return decorator



from swibots import (
    BotApp,
    MessageEvent,
    BotContext,
    CommandEvent,
    InlineMarkup,
    InlineKeyboardButton,
    CallbackQueryEvent,
    EmbeddedMedia,
    EmbedInlineField,
    Message,
    RegisterCommand,
    MediaUploadRequest,
    filters
)

import logging
import os

logging.basicConfig(level=logging.INFO)
import asyncio

app = BotApp(
  
    "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6OTc1LCJpc19ib3QiOnRydWUsImFjdGl2ZSI6dHJ1ZSwiaWF0IjoxNjg5MTU5NjEzLCJleHAiOjIzMjAzMTE2MTN9.lfLspX1PKUdOdKDPMFHe3PtGSycdUEnA95QbUahSB6k"
      #  "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6OTgyLCJpc19ib3QiOnRydWUsImFjdGl2ZSI6dHJ1ZSwiaWF0IjoxNjg5NDE0NTE3LCJleHAiOjIzMjA1NjY1MTd9.DNlqVeGHmlQIEKfj-H9SF9Hb654rMc48YLLWkc1fJoQ"
)
app.register_command([RegisterCommand("start", "Get start", True)])
app.register_command(
    [RegisterCommand("testcancel", "r", True)]
).register_command([
    RegisterCommand("cancel", "a")
])

global client
client = None

@app.on_callback_query()
async def eee(e):
    print(type(e.event.message.user_id))

@app.on_message(filters.community("30338819-7b57-4877-821f-b4ce2521614c"))
async def cce(e: BotContext[MessageEvent]):
#    print(e.event.message.replied_to)
    print(e.user.id, type(e.user.id))
    return
    await e.event.message.reply_text("Hi", inline_markup=InlineMarkup(
        [[
            InlineKeyboardButton("ij", callback_data="aa")
        ]]
    ))
    return
    print(e.event.message.replied_to, e.event.message.replied_to_id)

#    ct = await e.event.message.reply_text("https://google.com")
    return
#    m = await e.event.message.reply_text("Hello")
    c = await e.event.message.reply_text("hi")
    print(c.replied_to, c.replied_to_id)
    return
    print(m.replied_to_id, type(m.replied_to_id))
    print(m.replied_to)

#    print(await (await m.get_replied_message()).get_replied_message())

def humanbytes(size):
    if not size:
        return "0 B"
    for unit in ["", "K", "M", "G", "T"]:
        if size < 1024:
            break
        size /= 1024
    if isinstance(size, int):
        size = f"{size}{unit}B"
    elif isinstance(size, float):
        size = f"{size:.2f}{unit}B"
    return size

FILE =  r"C:\Users\Deves\Downloads\lineage-14.1-20170911-UNOFFICIAL-mocha.zip" # "../Switch.zip" # "test1.bin" # r"C:\Users\Deves\Downloads\5GB.bin" #"../Switch.zip" #r"C:\Users\Deves\Downloads\lineage-14.1-20170911-UNOFFICIAL-mocha.zip"

@app.on_command("cancel")
async def cance(e):
    global client
    client.cancel()
    await e.event.message.reply_text("cancelled!")

@app.on_command("start"#, filter=filters.community("30338819-7b57-4877-821f-b4ce2521614c")
                )
async def _eeee(e: BotContext[CommandEvent]):
#    print(e.event.message)
    m = e.event.message
    print(m)
    filesize = humanbytes(os.path.getsize(FILE))
    test = await m.reply_text("Processing")
    global last
    last = 0

    async def progress(c):
        print("Uploaded", humanbytes(c.readed), "Total", filesize)
        #global last
       # _is = last / c.readed > 10*1024*1024
       # last = c.readed
       # if _is:
        global client
        client = c.client
        await test.edit_text(
                f"Uploaded: {humanbytes(c.readed)}\nTotal: {filesize}"
        )#
#        if c.readed > 1024*2:
#         c.client.cancel()

    msg = await e.event.message.send("see", MediaUploadRequest(
        FILE,
        callback=progress,
#        block=
    ))
    print(msg)
    print("async")
    return



    async def progress(c):
        print("Uploaded", humanbytes(c.readed), "Total", filesize)
#        c.client.cancel()
 #       if c.readed > 1024*2:
 #       c.client.cancel()

    await e.event.message.send("see", MediaUploadRequest(
        FILE,
        callback=progress,
#        block=False
    ))
    print("async")

    return
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
    print(await e.enable_messages(m.community_id,
                            channel_id=m.channel_id,
                            group_id=m.group_id))
    return
    print(type(e.event.message.replied_to), 
          type(await e.event.message.get_replied_message()))
#    print(e.event.message)
#    await e.event.message.reply_text("Hi")

app.run()
exit()

async def main():
#    print(app.bot.user_name)
#    app.bot.
    await app.start()
    print(await app.get_user(972))
    return
    print(app.user, type(app.user))
    m = await app.get_message(799507)
    nm = m._prepare_response()
    nm.message = "Hello"
    chat = await m.reply(
        nm
    )
    print(chat.replied_to)
    await app.stop()
    return
    app.get_all_channels()
    print(await app.get_message(782235))
    app.list_restricted_users()
    app.get_bot_info()
    app.get_tournaments
    return
    await app.send_message(Message(
        app,
        community_id="28375c0f-ef39-485f-9aa6-8eeafab03a5c",
        group_id="640ee738-8ee9-4112-b894-5c4097cdfdad"
    ), media=MediaUploadRequest(
        path="docs/static/img/switch-logo.png",
        thumbnail="y.png"
    ),)
#    msg = Message(
#        app,
 #       group_id=""
  #  )
#    print(await app.get_bot_info(972))

#    print((await app.get_user(972)).name)
    return
    await app.send_message(Message(
        app,
        group_id="640ee738-8ee9-4112-b894-5c4097cdfdad",
        community_id="28375c0f-ef39-485f-9aa6-8eeafab03a5c",
        message="Hi",
        request_id=1
    ), media=MediaUploadRequest(
        path=".gitignore",
        description="OK"
    ))
 #   print(await app.get_messaging_enabled("f84cd816-cd4b-4e93-9202-695a6f9db55b", "bd3a805c-7987-4b39-a66d-f2afd6e6d1ec",))
  #  print(
   #     await app.get_messaging_enabled("f84cd816-cd4b-4e93-9202-695a6f9db55b", "bd3a805c-7987-4b39-a66d-f2afd6e6d1ec")
    #)
    
#    await app.enable_messages("")
  #  game = await app.update_leaderboard(
 #       "02d82f33-7f54-4538-a6e5-bea4956c809f", 973, 260
#    )
    #   print(game)
#    print(await app.get_global_leaderboard())
    return
    message = Message(app)
    message.message = "Hello"
    message.receiver_id = 972
    #    message.inline_markup = InlineMarkup([[
    #       InlineKeyboardButton("Test")
    #  ]])
    ok = await app.send_message(
        message,
        media=EmbeddedMedia(
            thumbnail="D:\Switch-Bots-Python-Library\docs\static\img\logo.png",
            description="J",
            inline_fields=[[EmbedInlineField("", "", "e")]],
        ),
    )
    await ok.edit_text(
        "hea",
        EmbeddedMedia(
            title="E",
            thumbnail="D:\Switch-Bots-Python-Library\docs\static\img\docusaurus.png",
            inline_fields=[[EmbedInlineField("", "", "c")]],
        ),
    )
    print(ok.id)
    return
    print(
        await ok.edit_text(
            None,
            EmbeddedMedia(
                thumbnail=None,
                description="HELLOOOOO",
                inline_fields=[[EmbedInlineField("", "", "Sorry")]],
            ),
        )
    )


#    print(ok)
# print(
#     (
#         await app.get_messaging_enabled(
#             "f84cd816-cd4b-4e93-9202-695a6f9db55b",
#             "bd3a805c-7987-4b39-a66d-f2afd6e6d1ec",
#         )
#     )[0].bot_id
# )


asyncio.run(main())
exit()


@app.on_message()
async def onMessage(ctx: BotContext[MessageEvent]):
    await ctx.event.message.send(
        EmbeddedMedia(
            thumbnail=None,
            header_icon="",
            title="a",
            description="a",
            inline_fields=[[EmbedInlineField("", "a", "aa")]],
        )
    )
    #    print(ctx.event.message)
    #    return
    x = await ctx.event.message.respond(
        "hi",
        inline_markup=InlineMarkup(
            [
                [
                    InlineKeyboardButton("Hi", callback_data="a"),
                    InlineKeyboardButton("C", callback_data="d"),
                ]
            ],
        ),
    )
    await x.edit_text("bye")


@app.on_callback_query()
async def ee(e: BotContext[CallbackQueryEvent]):
    print(e.event.callback_data)
    await e.event.message.send("hi")


@app.on_member_joined()
async def onJoin(ctx):
    print(ctx.event)


app.run()



import logging, mimetypes
from swibots.utils import ReadCallbackStream
from typing import TYPE_CHECKING, List, Optional
from urllib.parse import urlencode
from ..models import Sticker, StickerPack
from io import BytesIO

if TYPE_CHECKING:
    from swibots.api.chat import ChatClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/sticker"


class StickerController:
    """Stickers controller"""

    def __init__(self, client: "ChatClient"):
        self.client = client

    async def get_stickers(
        self, pack_id: str, limit: int = 30, offset: int = 0
    ) -> List[Sticker]:
        data = {"stickerPackId": pack_id, "offset": offset, "limit": limit}
        response = await self.client.get(f"{BASE_PATH}?{urlencode(data)}")
        return self.client.build_list(Sticker, response.data)

    async def create_sticker(
        self,
        sticker: str | BytesIO,
        name: str,
        description: str,
        emoji: str,
        pack_id: str,
    ) -> Sticker:
        file_name = sticker.name if isinstance(sticker, BytesIO) else sticker
        response = await self.client.post(
            BASE_PATH,
            form_data={
                "stickerPackId": pack_id,
                "linkedEmoji": emoji,
                "name": name,
                "description": description,
            },
            files={
                "uploadMediaRequest.file": (
                    file_name,
                    ReadCallbackStream(sticker),
                    mimetypes.guess_type(file_name)[0],
                )
            },
        )
        return self.client.build_object(Sticker, response.data)

    async def delete_sticker(self, sticker_id: int) -> bool:
        await self.client.delete(f"{BASE_PATH}?id={sticker_id}")
        return True

    # region

    async def create_sticker_pack(
        self, name: str, pack_type: str, access: str, thumb: str | BytesIO
    ) -> StickerPack:
        form_data = {
            "name": name,
            "packType": pack_type,
            "accessControl": access,
        }
        files = None
        if thumb:
            thumb_name = thumb.name if isinstance(thumb, BytesIO) else thumb
            files = {
                "uploadMediaRequest.file": (
                    thumb_name,
                    ReadCallbackStream(thumb),
                    mimetypes.guess_type(thumb_name)[0],
                )
            }
        response = await self.client.post(
            f"{BASE_PATH}/pack", form_data=form_data, files=files
        )
        return self.client.build_object(StickerPack, response.data)

    async def delete_sticker_pack(self, pack_id: int) -> bool:
        await self.client.delete(f"{BASE_PATH}/pack?id={pack_id}")
        return True

    async def search_sticker_packs(
        self, query: str, limit: int = 30, offset: int = 0
    ) -> List[StickerPack]:
        data = {"query": query, "offset": offset, "limit": limit}
        response = await self.client.get(f"{BASE_PATH}/pack/search?{urlencode(data)}")
        return self.client.build_list(StickerPack, response.data)

    async def get_all_sticker_packs(self, limit: int = 10, offset: int = 0):
        data = {"limit": limit, "offset": offset}
        response = await self.client.get(f"{BASE_PATH}/pack?{urlencode(data)}")
        return self.client.build_list(StickerPack, response.data)

    async def sort_stickers(
        self, pack: StickerPack, sorted_stickers: List[str] = None
    ) -> StickerPack:
        pack.stickers = sorted_stickers
        response = await self.client.post(
            f"{BASE_PATH}/pack/sort",
            data=pack.to_json_request(),
        )
        return self.client.build_object(StickerPack, response.data)

    async def install_sticker_pack(
        self, pack_id: str, community_id: str = None
    ) -> bool:
        data = {"stickerPackId": pack_id}
        if community_id:
            data["communityId"] = community_id
        response = await self.client.post(
            f"{BASE_PATH}/pack/install?{urlencode(data)}",
        )
        return True

    async def search_sticker_packs(
        self, query: str, limit: int = 10, offset: int = 0
    ) -> List[StickerPack]:
        response = await self.client.get(
            f"{BASE_PATH}/pack/search",
            data={"query": query, "limit": limit, "offset": offset},
        )
        return self.client.build_list(StickerPack, response.data)

    # endregion



from typing import List, Optional
from .base.switch_object import SwitchObject
from .api.community.models import QuestCategory, Quest
from .utils.types import JSONDict


class CommunityQuestResponse(SwitchObject):
    def __init__(
        self, categories: List[QuestCategory] = None, quests: List[Quest] = None
    ):
        super().__init__()
        self.categories = categories
        self.quests = quests

    def from_json(self, data: JSONDict = None) -> "CommunityQuestResponse":
        if data is not None:
            self.quests = [
                QuestCategory.build_from_json(quest) for quest in data.get("quests")
            ]
            self.categories = [
                QuestCategory.build_from_json(category)
                for category in data.get("availableCategories")
            ]
        return self



from typing import List
import swibots
from swibots.api.chat.models import Message


class GetMessages:
    async def get_messages(
        self: "swibots.ApiClient", user_id: int, limit: int = 100, offset: int = 0
    ) -> List[Message]:
        """Get messages

        Parameters:
            user_id (``int``, *optional*): The user id.
            limit (``int``, *optional*): The limit of messages to retrieve. Defaults to 100.
            offset (``int``, *optional*): The offset of messages to retrieve. Defaults to 0

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved

        This function does the same as :meth:`~switch.api.chat.controllers.MessageController.get_messages`.
        """
        return await self.chat_service.messages.get_messages(
            user_id, limit=limit, offset=offset
        )
